@@ -16,6 +16,7 @@
 
 
 #include <boost/format/format_class.hpp>
+#include <boost/format/exceptions.hpp>
 #include <boost/throw_exception.hpp>
 #include <boost/assert.hpp>
 
@@ -47,6 +48,7 @@ namespace detail {
 #if ! defined( BOOST_NO_LOCALE_ISDIGIT )
         return fac.is(std::ctype<Ch>::digit, c);
 # else
+        (void) fac;     // remove "unused parameter" warning
         using namespace std;
         return isdigit(c); 
 #endif 
@@ -130,6 +132,12 @@ namespace detail {
         bool in_brackets=false;
         Iter start0 = start;
         std::size_t fstring_size = last-start0+offset;
+
+        if(start>= last) { // empty directive : this is a trailing %
+                maybe_throw_exception(exceptions, start-start0 + offset, fstring_size);
+                return false;
+        }          
+          
         if(*start== const_or_not(fac).widen( '|')) {
             in_brackets=true;
             if( ++start >= last ) {
@@ -360,7 +368,10 @@ namespace detail {
             if( i1+1 >= buf.size() ) {
                 if(exceptions & bad_format_string_bit)
                     boost::throw_exception(bad_format_string(i1, buf.size() )); // must not end in ".. %"
-                else break; // stop there, ignore last '%'
+                else {
+                  ++num_items;
+                  break;
+                }
             }
             if(buf[i1+1] == buf[i1] ) {// escaped "%%"
                 i1+=2; continue; 
@@ -379,11 +390,7 @@ namespace detail {
     void append_string(String& dst, const String& src, 
                        const typename String::size_type beg, 
                        const typename String::size_type end) {
-#if !defined(BOOST_NO_STRING_APPEND)
         dst.append(src.begin()+beg, src.begin()+end);
-#else
-        dst += src.substr(beg, end-beg);
-#endif
     }
 
 } // detail namespace
@@ -429,8 +436,10 @@ namespace detail {
             }
             BOOST_ASSERT(  static_cast<unsigned int>(cur_item) < items_.size() || cur_item==0);
 
-            if(i1!=i0)
+            if(i1!=i0) {
                 io::detail::append_string(piece, buf, i0, i1);
+                i0=i1;
+            }
             ++i1;
             it = buf.begin()+i1;
             bool parse_ok = io::detail::parse_printf_directive(
@@ -462,7 +471,8 @@ namespace detail {
         if( !ordered_args) {
             if(max_argN >= 0 ) {  // dont mix positional with non-positionnal directives
                 if(exceptions() & io::bad_format_string_bit)
-                    boost::throw_exception(io::bad_format_string(max_argN, 0));
+                    boost::throw_exception(
+                        io::bad_format_string(static_cast<std::size_t>(max_argN), 0));
                 // else do nothing. => positionnal arguments are processed as non-positionnal
             }
             // set things like it would have been with positional directives :
