@@ -2,20 +2,21 @@
 //  Copyright (c) 2000-2002
 //  Joerg Walter, Mathias Koch
 //
-//  Permission to use, copy, modify, distribute and sell this software
-//  and its documentation for any purpose is hereby granted without fee,
-//  provided that the above copyright notice appear in all copies and
-//  that both that copyright notice and this permission notice appear
-//  in supporting documentation.  The authors make no representations
-//  about the suitability of this software for any purpose.
-//  It is provided "as is" without express or implied warranty.
+//  Distributed under the Boost Software License, Version 1.0. (See
+//  accompanying file LICENSE_1_0.txt or copy at
+//  http://www.boost.org/LICENSE_1_0.txt)
 //
 //  The authors gratefully acknowledge the support of
 //  GeNeSys mbH & Co. KG in producing this work.
 //
 
-#ifndef BOOST_UBLAS_OPERATION_BLOCKED_H
-#define BOOST_UBLAS_OPERATION_BLOCKED_H
+#ifndef _BOOST_UBLAS_OPERATION_BLOCKED_
+#define _BOOST_UBLAS_OPERATION_BLOCKED_
+
+#include <boost/numeric/ublas/traits.hpp>
+#include <boost/numeric/ublas/detail/vector_assign.hpp> // indexing_vector_assign
+#include <boost/numeric/ublas/detail/matrix_assign.hpp> // indexing_matrix_assign
+
 
 namespace boost { namespace numeric { namespace ublas {
 
@@ -36,7 +37,7 @@ namespace boost { namespace numeric { namespace ublas {
         vector<value_type> cv (v.size ());
         typedef typename type_traits<value_type>::real_type real_type;
         real_type verrorbound (norm_1 (v) + norm_1 (e1) * norm_1 (e2));
-        indexing_vector_assign (scalar_assign<typename vector<value_type>::reference, value_type> (), cv, prod (e1, e2));
+        indexing_vector_assign<scalar_assign> (cv, prod (e1, e2));
 #endif
         size_type i_size = e1 ().size1 ();
         size_type j_size = BOOST_UBLAS_SAME (e1 ().size2 (), e2 ().size ());
@@ -91,7 +92,7 @@ namespace boost { namespace numeric { namespace ublas {
         vector<value_type> cv (v.size ());
         typedef typename type_traits<value_type>::real_type real_type;
         real_type verrorbound (norm_1 (v) + norm_1 (e1) * norm_1 (e2));
-        indexing_vector_assign (scalar_assign<typename vector<value_type>::reference, value_type> (), cv, prod (e1, e2));
+        indexing_vector_assign<scalar_assign> (cv, prod (e1, e2));
 #endif
         size_type i_size = BOOST_UBLAS_SAME (e1 ().size (), e2 ().size1 ());
         size_type j_size = e2 ().size2 ();
@@ -146,7 +147,7 @@ namespace boost { namespace numeric { namespace ublas {
         matrix<value_type, row_major> cm (m.size1 (), m.size2 ());
         typedef typename type_traits<value_type>::real_type real_type;
         real_type merrorbound (norm_1 (m) + norm_1 (e1) * norm_1 (e2));
-        indexing_matrix_assign (scalar_assign<typename matrix<value_type, row_major>::reference, value_type> (), cm, prod (e1, e2), row_major_tag ());
+        indexing_matrix_assign<scalar_assign> (cm, prod (e1, e2), row_major_tag ());
         disable_type_check<bool>::value = true;
 #endif
         size_type i_size = e1 ().size1 ();
@@ -207,7 +208,7 @@ namespace boost { namespace numeric { namespace ublas {
         matrix<value_type, column_major> cm (m.size1 (), m.size2 ());
         typedef typename type_traits<value_type>::real_type real_type;
         real_type merrorbound (norm_1 (m) + norm_1 (e1) * norm_1 (e2));
-        indexing_matrix_assign (scalar_assign<typename matrix<value_type, column_major>::reference, value_type> (), cm, prod (e1, e2), column_major_tag ());
+        indexing_matrix_assign<scalar_assign> (cm, prod (e1, e2), column_major_tag ());
         disable_type_check<bool>::value = true;
 #endif
         size_type i_size = e1 ().size1 ();
