@@ -3,7 +3,7 @@
 
 /////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
 // MS compatible compilers support #pragma once
-#if defined(_MSC_VER) && (_MSC_VER >= 1020)
+#if defined(_MSC_VER)
 # pragma once
 #endif
 
@@ -20,7 +20,7 @@
 
 #include <typeinfo>
 #include <cstdarg>
-#include <cassert>
+#include <boost/assert.hpp>
 #include <boost/config.hpp>
 
 #include <boost/static_assert.hpp>
@@ -28,26 +28,38 @@
 #include <boost/type_traits/is_polymorphic.hpp>
 #include <boost/type_traits/remove_const.hpp>
 
+#include <boost/serialization/config.hpp>
 #include <boost/serialization/singleton.hpp>
 #include <boost/serialization/extended_type_info.hpp>
 #include <boost/serialization/factory.hpp>
 
+// hijack serialization access
+#include <boost/serialization/access.hpp>
+
+#include <boost/mpl/if.hpp>
+
 #include <boost/config/abi_prefix.hpp> // must be the last header
+
 #ifdef BOOST_MSVC
 #  pragma warning(push)
-#  pragma warning(disable : 4251 4231 4660 4275)
+#  pragma warning(disable : 4251 4231 4660 4275 4511 4512)
 #endif
 
 namespace boost {
 namespace serialization {
-namespace detail {
+namespace typeid_system {
 
 class BOOST_SERIALIZATION_DECL(BOOST_PP_EMPTY()) extended_type_info_typeid_0 : 
     public extended_type_info
 {
+    virtual const char * get_debug_info() const {
+        if(static_cast<const std::type_info *>(0) == m_ti)
+            return static_cast<const char *>(0);
+        return m_ti->name();
+    }
 protected:
     const std::type_info * m_ti;
-    extended_type_info_typeid_0();
+    extended_type_info_typeid_0(const char * key);
     ~extended_type_info_typeid_0();
     void type_register(const std::type_info & ti);
     void type_unregister();
@@ -63,20 +75,22 @@ public:
     }
 };
 
-} // namespace detail
+} // typeid_system
 
 template<class T>
 class extended_type_info_typeid : 
-    public detail::extended_type_info_typeid_0,
-    public singleton<extended_type_info_typeid<T> >
+    public typeid_system::extended_type_info_typeid_0,
+    public singleton<extended_type_info_typeid< T > >
 {
 public:
     extended_type_info_typeid() :
-        detail::extended_type_info_typeid_0()
+        typeid_system::extended_type_info_typeid_0(get_key())
     {
         type_register(typeid(T));
+        key_register();
     }
     ~extended_type_info_typeid(){
+        key_unregister();
         type_unregister();
     }
     // get the eti record for the true type of this record
@@ -85,42 +99,41 @@ public:
     get_derived_extended_type_info(const T & t) const {
         // note: this implementation - based on usage of typeid (rtti)
         // only does something if the class has at least one virtual function.
-        BOOST_STATIC_WARNING(boost::is_polymorphic<T>::value);
+        BOOST_STATIC_WARNING(boost::is_polymorphic< T >::value);
         return 
-            detail::extended_type_info_typeid_0::get_extended_type_info(
+            typeid_system::extended_type_info_typeid_0::get_extended_type_info(
                 typeid(t)
             );
     }
-    void * construct(unsigned int count, ...) const{
+    const char * get_key() const {
+        return boost::serialization::guid< T >();
+    }
+    virtual void * construct(unsigned int count, ...) const{
         // count up the arguments
         std::va_list ap;
         va_start(ap, count);
         switch(count){
         case 0:
-            return factory<boost::remove_const<T>, 0>(ap);
+            return factory<typename boost::remove_const< T >::type, 0>(ap);
         case 1:
-            return factory<boost::remove_const<T>, 1>(ap);
+            return factory<typename boost::remove_const< T >::type, 1>(ap);
         case 2:
-            return factory<boost::remove_const<T>, 2>(ap);
+            return factory<typename boost::remove_const< T >::type, 2>(ap);
         case 3:
-            return factory<boost::remove_const<T>, 3>(ap);
+            return factory<typename boost::remove_const< T >::type, 3>(ap);
         case 4:
-            return factory<boost::remove_const<T>, 4>(ap);
+            return factory<typename boost::remove_const< T >::type, 4>(ap);
         default:
-            assert(false); // too many arguments
+            BOOST_ASSERT(false); // too many arguments
             // throw exception here?
             return NULL;
         }
     }
-    void destroy(void const * const p) const {
-        // the only current usage of extended type info is in the
-        // serialization library.  The statement below requires
-        // that destructor of type T be public and this creates
-        // a problem for some users.  So, for now, comment this
-        // out 
-        //delete static_cast<T const *>(p);
-        // and trap any attempt to invoke this function
-        assert(false);
+    virtual void destroy(void const * const p) const {
+        boost::serialization::access::destroy(
+            static_cast<T const *>(p)
+        );
+        //delete static_cast<T const * const>(p);
     }
 };
 
@@ -137,8 +150,8 @@ public:
     namespace serialization {
     template<class T>
     struct extended_type_info_impl {
-        typedef BOOST_DEDUCED_TYPENAME 
-            boost::serialization::extended_type_info_typeid<T> type;
+        typedef typename 
+            boost::serialization::extended_type_info_typeid< T > type;
     };
     } // namespace serialization
     } // namespace boost
