@@ -2,7 +2,7 @@
 // host_name.cpp
 // ~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2008 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2015 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -16,7 +16,6 @@
 // Test that header file is self-contained.
 #include <boost/asio/ip/host_name.hpp>
 
-#include <boost/asio.hpp>
 #include "../unit_test.hpp"
 
 //------------------------------------------------------------------------------
@@ -49,9 +48,8 @@ void test()
 
 //------------------------------------------------------------------------------
 
-test_suite* init_unit_test_suite(int, char*[])
-{
-  test_suite* test = BOOST_TEST_SUITE("ip/host_name");
-  test->add(BOOST_TEST_CASE(&ip_host_name_compile::test));
-  return test;
-}
+BOOST_ASIO_TEST_SUITE
+(
+  "ip/host_name",
+  BOOST_ASIO_TEST_CASE(ip_host_name_compile::test)
+)
