@@ -179,7 +179,7 @@ template < typename Graph, typename TimePropertyMap >
   S.push(std::make_pair(time_stamp, src));
   while (!S.empty()) {
     Vertex x;
-    tie(time_stamp, x) = S.top();
+    boost::tie(time_stamp, x) = S.top();
     put(time_map, x, time_stamp);
     // all vertices have been visited, success!
     if (time_stamp == num_vertices(g) - 1)
@@ -187,7 +187,7 @@ template < typename Graph, typename TimePropertyMap >
 
     bool deadend = true;
     typename graph_traits < Graph >::adjacency_iterator i, end;
-    for (tie(i, end) = adjacent_vertices(x, g); i != end; ++i)
+    for (boost::tie(i, end) = adjacent_vertices(x, g); i != end; ++i)
       if (get(time_map, *i) == -1) {
         S.push(std::make_pair(time_stamp + 1, *i));
         deadend = false;
@@ -196,11 +196,11 @@ template < typename Graph, typename TimePropertyMap >
     if (deadend) {
       put(time_map, x, -1);
       S.pop();
-      tie(time_stamp, x) = S.top();
+      boost::tie(time_stamp, x) = S.top();
       while (get(time_map, x) != -1) {  // unwind stack to last unexplored vertex
         put(time_map, x, -1);
         S.pop();
-        tie(time_stamp, x) = S.top();
+        boost::tie(time_stamp, x) = S.top();
       }
     }
 
@@ -213,7 +213,7 @@ number_of_successors(Vertex x, Graph & g, TimePropertyMap time_map)
 {
   int s_x = 0;
   typename graph_traits < Graph >::adjacency_iterator i, end;
-  for (tie(i, end) = adjacent_vertices(x, g); i != end; ++i)
+  for (boost::tie(i, end) = adjacent_vertices(x, g); i != end; ++i)
     if (get(time_map, *i) == -1)
       ++s_x;
   return s_x;
@@ -232,7 +232,7 @@ template < typename Graph, typename TimePropertyMap >
   S.push(std::make_pair(time_stamp, src));
   while (!S.empty()) {
     Vertex x;
-    tie(time_stamp, x) = S.top();
+    boost::tie(time_stamp, x) = S.top();
     put(time_map, x, time_stamp);
     // all vertices have been visited, success!
     if (time_stamp == num_vertices(g) - 1)
@@ -242,7 +242,7 @@ template < typename Graph, typename TimePropertyMap >
     std::priority_queue < P, std::vector < P >, compare_first > Q;
     typename graph_traits < Graph >::adjacency_iterator i, end;
     int num_succ;
-    for (tie(i, end) = adjacent_vertices(x, g); i != end; ++i)
+    for (boost::tie(i, end) = adjacent_vertices(x, g); i != end; ++i)
       if (get(time_map, *i) == -1) {
         num_succ = number_of_successors(*i, g, time_map);
         Q.push(std::make_pair(num_succ, *i));
@@ -250,17 +250,17 @@ template < typename Graph, typename TimePropertyMap >
     bool deadend = Q.empty();
     // move vertices from local priority queue to the stack
     for (; !Q.empty(); Q.pop()) {
-      tie(num_succ, x) = Q.top();
+      boost::tie(num_succ, x) = Q.top();
       S.push(std::make_pair(time_stamp + 1, x));
     }
     if (deadend) {
       put(time_map, x, -1);
       S.pop();
-      tie(time_stamp, x) = S.top();
+      boost::tie(time_stamp, x) = S.top();
       while (get(time_map, x) != -1) {  // unwind stack to last unexplored vertex
         put(time_map, x, -1);
         S.pop();
-        tie(time_stamp, x) = S.top();
+        boost::tie(time_stamp, x) = S.top();
       }
     }
 
