@@ -1,6 +1,6 @@
 ////////////////////////////////////////
 //
-// (C) Copyright Ion Gaztanaga 2006. Distributed under the Boost
+// (C) Copyright Ion Gaztanaga 2006-2012. Distributed under the Boost
 // Software License, Version 1.0. (See accompanying file
 // LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 //
@@ -18,7 +18,7 @@
 #include <vector>
 #include <functional>
 #include "print_container.hpp"
-#include <boost/interprocess/detail/move.hpp>
+#include <boost/move/utility_core.hpp>
 #include <string>
 #include "get_process_id_name.hpp"
 
@@ -35,8 +35,10 @@ struct push_data_function
       typedef typename MyShmList::value_type IntType;
       for(int i = 0; i < max; ++i){
          IntType move_me(i);
-         shmlist->push_back(boost::interprocess::move(move_me));
+         shmlist->push_back(boost::move(move_me));
          stdlist->push_back(i);
+         shmlist->push_back(IntType(i));
+         stdlist->push_back(int(i));
       }
       if(!CheckEqualContainers(shmlist, stdlist))
          return 1;
@@ -53,8 +55,10 @@ struct push_data_function<false>
       typedef typename MyShmList::value_type IntType;
       for(int i = 0; i < max; ++i){
          IntType move_me(i);
-         shmlist->push_front(boost::interprocess::move(move_me));
+         shmlist->push_front(boost::move(move_me));
          stdlist->push_front(i);
+         shmlist->push_front(IntType(i));
+         stdlist->push_front(int(i));
       }
       if(!CheckEqualContainers(shmlist, stdlist))
          return 1;
@@ -115,7 +119,7 @@ int list_test (bool copied_allocators_equal = true)
 
       MyStdList *stdlist = new MyStdList;
 
-      if(push_data_t::execute(max, shmlist, stdlist)){
+      if(push_data_t::execute(max/2, shmlist, stdlist)){
          return 1;
       }
 
@@ -135,14 +139,14 @@ int list_test (bool copied_allocators_equal = true)
          IntType aux_vect[50];
          for(int i = 0; i < 50; ++i){
             IntType move_me(-1);
-            aux_vect[i] = boost::interprocess::move(move_me);
+            aux_vect[i] = boost::move(move_me);
          }
          int aux_vect2[50];
          for(int i = 0; i < 50; ++i){
             aux_vect2[i] = -1;
          }
-         shmlist->assign(boost::interprocess::make_move_iterator(&aux_vect[0])
-                        ,boost::interprocess::make_move_iterator(&aux_vect[50]));
+         shmlist->assign(::boost::make_move_iterator(&aux_vect[0])
+                        ,::boost::make_move_iterator(&aux_vect[50]));
          stdlist->assign(&aux_vect2[0], &aux_vect2[50]);
          if(!CheckEqualContainers(shmlist, stdlist)) return 1;
       }
@@ -165,15 +169,15 @@ int list_test (bool copied_allocators_equal = true)
          IntType aux_vect[50];
          for(int i = 0; i < 50; ++i){
             IntType move_me(-1);
-            aux_vect[i] = boost::interprocess::move(move_me);
+            aux_vect[i] = boost::move(move_me);
          }
          int aux_vect2[50];
          for(int i = 0; i < 50; ++i){
             aux_vect2[i] = -1;
          }
          shmlist->insert(shmlist->begin()
-                        ,boost::interprocess::make_move_iterator(&aux_vect[0])
-                        ,boost::interprocess::make_move_iterator(&aux_vect[50]));
+                        ,::boost::make_move_iterator(&aux_vect[0])
+                        ,::boost::make_move_iterator(&aux_vect[50]));
          stdlist->insert(stdlist->begin(), &aux_vect2[0], &aux_vect2[50]);
       }
 
@@ -198,7 +202,7 @@ int list_test (bool copied_allocators_equal = true)
       if(!CheckEqualContainers(shmlist, stdlist))
          return 1;
 
-      if(push_data_t::execute(max, shmlist, stdlist)){
+      if(push_data_t::execute(max/2, shmlist, stdlist)){
          return 1;
       }
       {
@@ -207,7 +211,7 @@ int list_test (bool copied_allocators_equal = true)
 
          int listsize = (int)shmlist->size();
 
-         if(push_data_t::execute(listsize, shmlist, stdlist)){
+         if(push_data_t::execute(listsize/2, shmlist, stdlist)){
             return 1;
          }
 
@@ -215,16 +219,16 @@ int list_test (bool copied_allocators_equal = true)
             shmlist->splice(shmlist->begin(), othershmlist);
             stdlist->splice(stdlist->begin(), otherstdlist);
             if(!CheckEqualContainers(shmlist, stdlist))
-               return 1;   
+               return 1;
          }
 
          listsize = (int)shmlist->size();
 
-         if(push_data_t::execute(listsize, shmlist, stdlist)){
+         if(push_data_t::execute(listsize/2, shmlist, stdlist)){
             return 1;
          }
 
-         if(push_data_t::execute(listsize, &othershmlist, &otherstdlist)){
+         if(push_data_t::execute(listsize/2, &othershmlist, &otherstdlist)){
             return 1;
          }
 
@@ -244,6 +248,13 @@ int list_test (bool copied_allocators_equal = true)
             if(!CheckEqualContainers(shmlist, stdlist))
                return 1;
          }
+
+         for(int i = 0; i < max; ++i){
+            shmlist->insert(shmlist->begin(), IntType(i));
+            stdlist->insert(stdlist->begin(), int(i));
+         }
+         if(!CheckEqualContainers(shmlist, stdlist))
+            return 1;
       }
 
       segment.template destroy<MyShmList>("MyList");
