@@ -1,7 +1,7 @@
 //////////////////////////////////////////////////////////////////////////////
 //
-// (C) Copyright Peter Dimov 2002-2005.
-// (C) Copyright Ion Gaztanaga 2006-2007.
+// (C) Copyright Peter Dimov 2002-2005, 2007.
+// (C) Copyright Ion Gaztanaga 2006-2012.
 // Distributed under the Boost Software License, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at
 // http://www.boost.org/LICENSE_1_0.txt)
@@ -21,10 +21,14 @@
 #include <boost/interprocess/containers/vector.hpp>
 #include <boost/interprocess/smart_ptr/deleter.hpp>
 #include <boost/interprocess/smart_ptr/scoped_ptr.hpp>
-#include <boost/detail/lightweight_test.hpp>
+#include <boost/core/lightweight_test.hpp>
 #include <string>
 #include "get_process_id_name.hpp"
 
+
+#include <boost/interprocess/sync/upgradable_lock.hpp>
+#include <boost/interprocess/sync/interprocess_upgradable_mutex.hpp>
+
 using namespace boost::interprocess;
 
 class base_class
@@ -49,7 +53,6 @@ int simple_test()
    typedef deleter<base_class, managed_shared_memory::segment_manager>
       base_deleter_t;
    typedef shared_ptr<base_class, base_class_allocator, base_deleter_t>    base_shared_ptr;
-   typedef weak_ptr<base_class, base_class_allocator, base_deleter_t>      base_weak_ptr;
 
    std::string process_name;
    test::get_process_id_name(process_name);
@@ -59,16 +62,16 @@ int simple_test()
       managed_shared_memory shmem(create_only, process_name.c_str(), 10000);
 
       {
-         base_shared_ptr s_ptr(base_shared_ptr::pointer(0), 
-                           base_class_allocator(shmem.get_segment_manager()), 
+         base_shared_ptr s_ptr(base_shared_ptr::pointer(0),
+                           base_class_allocator(shmem.get_segment_manager()),
                            base_deleter_t(shmem.get_segment_manager()));
 
-         base_shared_ptr s_ptr2(shmem.construct<base_class>("base_class")(), 
-                              base_class_allocator(shmem.get_segment_manager()), 
+         base_shared_ptr s_ptr2(shmem.construct<base_class>("base_class")(),
+                              base_class_allocator(shmem.get_segment_manager()),
                               base_deleter_t(shmem.get_segment_manager()));
 
-         base_shared_ptr s_ptr3(offset_ptr<derived_class>(shmem.construct<derived_class>("derived_class")()), 
-                              base_class_allocator(shmem.get_segment_manager()), 
+         base_shared_ptr s_ptr3(offset_ptr<derived_class>(shmem.construct<derived_class>("derived_class")()),
+                              base_class_allocator(shmem.get_segment_manager()),
                               base_deleter_t(shmem.get_segment_manager()));
 
          if(s_ptr3.get_deleter()   == 0){
@@ -107,20 +110,20 @@ int string_shared_ptr_vector_insertion_test()
       string_allocator_t;
 
    //A deleter for shared_ptr<> that erases a shared memory string
-   typedef deleter<string_t, managed_shared_memory::segment_manager> 
+   typedef deleter<string_t, managed_shared_memory::segment_manager>
       string_deleter_t;
 
    //A shared pointer that points to a shared memory string and its instantiation
    typedef shared_ptr<string_t, string_allocator_t, string_deleter_t> string_shared_ptr_t;
 
-   //An allocator for shared pointers to a string in shared memory 
+   //An allocator for shared pointers to a string in shared memory
    typedef allocator<string_shared_ptr_t, managed_shared_memory::segment_manager>
       string_shared_ptr_allocator_t;
 
    //A weak pointer that points to a shared memory string and its instantiation
    typedef weak_ptr<string_t, string_allocator_t, string_deleter_t> string_weak_ptr_t;
 
-   //An allocator for weak pointers to a string in shared memory 
+   //An allocator for weak pointers to a string in shared memory
    typedef allocator<string_weak_ptr_t, managed_shared_memory::segment_manager >
       string_weak_ptr_allocator_t;
 
@@ -140,7 +143,7 @@ int string_shared_ptr_vector_insertion_test()
    {
       managed_shared_memory shmem(create_only, process_name.c_str(), 20000);
 
-      {  
+      {
          const int NumElements = 100;
          //Construct the allocator of strings
          string_allocator_t string_allocator(shmem.get_segment_manager());
@@ -168,6 +171,8 @@ int string_shared_ptr_vector_insertion_test()
          //Now fill a shared memory vector of shared_ptrs to a string
          string_shared_ptr_vector_t my_sharedptr_vector(string_shared_ptr_allocator);
          my_sharedptr_vector.insert(my_sharedptr_vector.begin(), NumElements, string_shared_ptr);
+         //Insert in the middle to test movability
+         my_sharedptr_vector.insert(my_sharedptr_vector.begin() + my_sharedptr_vector.size()/2, NumElements, string_shared_ptr);
          //Now check the shared count is the objects contained in the
          //vector plus string_shared_ptr
          if(string_shared_ptr.use_count() != static_cast<long>(my_sharedptr_vector.size()+1)){
@@ -196,7 +201,7 @@ int string_shared_ptr_vector_insertion_test()
          }
          //Now fill a vector of weak_ptr-s
          string_weak_ptr_vector_t my_weakptr_vector(string_weak_ptr_allocator);
-         my_weakptr_vector.insert(my_weakptr_vector.begin(), NumElements, string_weak_ptr);      
+         my_weakptr_vector.insert(my_weakptr_vector.begin(), NumElements, string_weak_ptr);
          //The shared count should remain the same
          if(string_shared_ptr.use_count() != static_cast<long>(my_sharedptr_vector.size()+1)){
             return 1;
@@ -303,28 +308,28 @@ int * get_object()
 void release_object(int * p)
 {  BOOST_TEST(p == &cnt);  --cnt;   }
 
-template<class T, class A, class D> 
+template<class T, class A, class D>
 void test_is_X(shared_ptr<T, A, D> const & p)
 {
    BOOST_TEST(p->id() == 1);
    BOOST_TEST((*p).id() == 1);
 }
 
-template<class T, class A, class D> 
+template<class T, class A, class D>
 void test_is_X(weak_ptr<T, A, D> const & p)
 {
    BOOST_TEST(p.get() != 0);
    BOOST_TEST(p.get()->id() == 1);
 }
 
-template<class T, class A, class D> 
+template<class T, class A, class D>
 void test_is_Y(shared_ptr<T, A, D> const & p)
 {
    BOOST_TEST(p->id() == 2);
    BOOST_TEST((*p).id() == 2);
 }
 
-template<class T, class A, class D> 
+template<class T, class A, class D>
 void test_is_Y(weak_ptr<T, A, D> const & p)
 {
    shared_ptr<T, A, D> q = p.lock();
@@ -332,7 +337,7 @@ void test_is_Y(weak_ptr<T, A, D> const & p)
    BOOST_TEST(q->id() == 2);
 }
 
-template<class T, class T2> 
+template<class T, class T2>
 void test_eq(T const & a, T2 const & b)
 {
    BOOST_TEST(a == b);
@@ -341,7 +346,7 @@ void test_eq(T const & a, T2 const & b)
    BOOST_TEST(!(b < a));
 }
 
-template<class T, class T2> 
+template<class T, class T2>
 void test_ne(T const & a, T2 const & b)
 {
    BOOST_TEST(!(a == b));
@@ -536,6 +541,76 @@ int basic_shared_ptr_test()
    return boost::report_errors();
 }
 
+struct alias_tester
+{
+    int v_;
+
+    explicit alias_tester( int v ): v_( v )
+    {
+    }
+
+    ~alias_tester()
+    {
+        v_ = 0;
+    }
+};
+
+void test_alias()
+{
+   typedef allocator<void, managed_shared_memory::segment_manager>
+      v_allocator_t;
+
+   typedef deleter<int, managed_shared_memory::segment_manager>
+      int_deleter_t;
+
+   typedef shared_ptr<int, v_allocator_t, int_deleter_t> int_shared_ptr;
+   typedef shared_ptr<const int, v_allocator_t, int_deleter_t> const_int_shared_ptr;
+
+   std::string process_name;
+   test::get_process_id_name(process_name);
+
+   shared_memory_object::remove(process_name.c_str());
+   {
+      managed_shared_memory shmem(create_only, process_name.c_str(), 10000);
+
+      {
+         int m = 0;
+         int_shared_ptr p;
+         int_shared_ptr p2( p, &m );
+
+         BOOST_TEST( ipcdetail::to_raw_pointer(p2.get()) == &m );
+         BOOST_TEST( p2? true: false );
+         BOOST_TEST( !!p2 );
+         BOOST_TEST( p2.use_count() == p.use_count() );
+         BOOST_TEST( !( p < p2 ) && !( p2 < p ) );
+
+         p2.reset( p, static_cast<int*>(0) );
+
+         BOOST_TEST( p2.get() == 0 );
+
+         BOOST_TEST( p2? false: true );
+         BOOST_TEST( !p2 );
+         BOOST_TEST( p2.use_count() == p.use_count() );
+         BOOST_TEST( !( p < p2 ) && !( p2 < p ) );
+      }
+
+      {
+         int m = 0;
+         int_shared_ptr p(make_managed_shared_ptr
+            (shmem.construct<int>(anonymous_instance)(), shmem));
+         const_int_shared_ptr p2( p, &m );
+
+         BOOST_TEST( ipcdetail::to_raw_pointer(p2.get()) == &m );
+         BOOST_TEST( p2? true: false );
+         BOOST_TEST( !!p2 );
+         BOOST_TEST( p2.use_count() == p.use_count() );
+         BOOST_TEST( !( p < p2 ) && !( p2 < p ) );
+         int_shared_ptr p_nothrow(make_managed_shared_ptr
+            (shmem.construct<int>(anonymous_instance)(), shmem, std::nothrow));
+      }
+   }
+   shared_memory_object::remove(process_name.c_str());
+}
 
 int main()
 {
@@ -548,7 +623,8 @@ int main()
    if(0 != basic_shared_ptr_test())
       return 1;
 
-   return 0;
+   test_alias();
 }
 
 #include <boost/interprocess/detail/config_end.hpp>
+
