@@ -55,6 +55,7 @@ void instantiate(RealType)
    using namespace boost::math;
    using namespace boost::math::concepts;
 #ifdef TEST_GROUP_1
+   function_requires<DistributionConcept<arcsine_distribution<RealType> > >();
    function_requires<DistributionConcept<bernoulli_distribution<RealType> > >();
    function_requires<DistributionConcept<beta_distribution<RealType> > >();
    function_requires<DistributionConcept<binomial_distribution<RealType> > >();
@@ -66,6 +67,7 @@ void instantiate(RealType)
    function_requires<DistributionConcept<gamma_distribution<RealType> > >();
    function_requires<DistributionConcept<geometric_distribution<RealType> > >();
    function_requires<DistributionConcept<hypergeometric_distribution<RealType> > >();
+   function_requires<DistributionConcept<hyperexponential_distribution<RealType> > >();
    function_requires<DistributionConcept<inverse_chi_squared_distribution<RealType> > >();
    function_requires<DistributionConcept<inverse_gamma_distribution<RealType> > >();
    function_requires<DistributionConcept<inverse_gaussian_distribution<RealType> > >();
@@ -89,6 +91,7 @@ void instantiate(RealType)
 #endif
 #ifndef BOOST_MATH_INSTANTIATE_MINIMUM
 #ifdef TEST_GROUP_2
+   function_requires<DistributionConcept<arcsine_distribution<RealType> > >();
    function_requires<DistributionConcept<bernoulli_distribution<RealType, test_policy> > >();
    function_requires<DistributionConcept<beta_distribution<RealType, test_policy> > >();
    function_requires<DistributionConcept<binomial_distribution<RealType, test_policy> > >();
@@ -122,6 +125,7 @@ void instantiate(RealType)
    function_requires<DistributionConcept<weibull_distribution<RealType, test_policy> > >();
 #endif
 #ifdef TEST_GROUP_3
+   function_requires<DistributionConcept<dist_test::arcsine > >();
    function_requires<DistributionConcept<dist_test::bernoulli > >();
    function_requires<DistributionConcept<dist_test::beta > >();
    function_requires<DistributionConcept<dist_test::binomial > >();
@@ -165,6 +169,8 @@ void instantiate(RealType)
    boost::math::lgamma(v1);
    boost::math::lgamma(v1, &i);
    boost::math::digamma(v1);
+   boost::math::trigamma(v1);
+   boost::math::polygamma(i, v1);
    boost::math::tgamma_ratio(v1, v2);
    boost::math::tgamma_delta_ratio(v1, v2);
    boost::math::factorial<RealType>(i);
@@ -198,6 +204,7 @@ void instantiate(RealType)
    boost::math::ibetac_invb(v1, v2, v3);
    boost::math::gamma_p_derivative(v2, v3);
    boost::math::ibeta_derivative(v1, v2, v3);
+   boost::math::binomial_coefficient<RealType>(i, i);
    (boost::math::fpclassify)(v1);
    (boost::math::isfinite)(v1);
    (boost::math::isnormal)(v1);
@@ -232,9 +239,14 @@ void instantiate(RealType)
    boost::math::ellint_2(v1, v2);
    boost::math::ellint_3(v1, v2);
    boost::math::ellint_3(v1, v2, v3);
+   boost::math::ellint_d(v1);
+   boost::math::ellint_d(v1, v2);
+   boost::math::jacobi_zeta(v1, v2);
+   boost::math::heuman_lambda(v1, v2);
    boost::math::ellint_rc(v1, v2);
    boost::math::ellint_rd(v1, v2, v3);
    boost::math::ellint_rf(v1, v2, v3);
+   boost::math::ellint_rg(v1, v2, v3);
    boost::math::ellint_rj(v1, v2, v3, v1);
    boost::math::jacobi_elliptic(v1, v2, &v1, &v2);
    boost::math::jacobi_cd(v1, v2);
@@ -269,6 +281,18 @@ void instantiate(RealType)
    boost::math::sph_bessel(i, 1);
    boost::math::sph_neumann(i, v2);
    boost::math::sph_neumann(i, i);
+   boost::math::cyl_neumann_prime(v1, v2);
+   boost::math::cyl_neumann_prime(i, v2);
+   boost::math::cyl_bessel_j_prime(v1, v2);
+   boost::math::cyl_bessel_j_prime(i, v2);
+   boost::math::cyl_bessel_i_prime(v1, v2);
+   boost::math::cyl_bessel_i_prime(i, v2);
+   boost::math::cyl_bessel_k_prime(v1, v2);
+   boost::math::cyl_bessel_k_prime(i, v2);
+   boost::math::sph_bessel_prime(i, v2);
+   boost::math::sph_bessel_prime(i, 1);
+   boost::math::sph_neumann_prime(i, v2);
+   boost::math::sph_neumann_prime(i, i);
    boost::math::cyl_bessel_j_zero(v1, i);
    boost::math::cyl_bessel_j_zero(v1, i, i, oi);
    boost::math::cyl_neumann_zero(v1, i);
@@ -321,6 +345,13 @@ void instantiate(RealType)
    boost::math::float_next(v1);
    boost::math::float_prior(v1);
    boost::math::float_distance(v1, v1);
+
+   boost::math::unchecked_bernoulli_b2n<RealType>(i);
+   boost::math::bernoulli_b2n<RealType>(i);
+   boost::math::bernoulli_b2n<RealType>(i, i, &v1);
+   boost::math::tangent_t2n<RealType>(i);
+   boost::math::tangent_t2n<RealType>(i, i, &v1);
+
 #endif
 #ifdef TEST_GROUP_9
    //
@@ -331,6 +362,8 @@ void instantiate(RealType)
    boost::math::lgamma(v1 * 1);
    boost::math::lgamma(v1 * 1, &i);
    boost::math::digamma(v1 * 1);
+   boost::math::trigamma(v1 * 1);
+   boost::math::polygamma(i, v1 * 1);
    boost::math::tgamma_ratio(v1 * 1, v2 + 0);
    boost::math::tgamma_delta_ratio(v1 * 1, v2 + 0);
    boost::math::factorial<RealType>(i);
@@ -401,7 +434,12 @@ void instantiate(RealType)
    boost::math::ellint_rc(v1 * 1, v2 + 0);
    boost::math::ellint_rd(v1 * 1, v2 + 0, v3 / 1);
    boost::math::ellint_rf(v1 * 1, v2 + 0, v3 / 1);
+   boost::math::ellint_rg(v1 * 1, v2 + 0, v3 / 1);
    boost::math::ellint_rj(v1 * 1, v2 + 0, v3 / 1, v1 * 1);
+   boost::math::ellint_d(v1 * 1);
+   boost::math::ellint_d(v1 * 1, v2 + 0);
+   boost::math::jacobi_zeta(v1 * 1, v2 + 0);
+   boost::math::heuman_lambda(v1 * 1, v2 + 0);
    boost::math::jacobi_elliptic(v1 * 1, v2 + 0, &v1, &v2);
    boost::math::jacobi_cd(v1 * 1, v2 + 0);
    boost::math::jacobi_cn(v1 * 1, v2 + 0);
@@ -435,6 +473,18 @@ void instantiate(RealType)
    boost::math::sph_bessel(i, 1);
    boost::math::sph_neumann(i, v2 * 1);
    boost::math::sph_neumann(i, i);
+   boost::math::cyl_neumann_prime(v1 * 1, v2 + 0);
+   boost::math::cyl_neumann_prime(i, v2 * 1);
+   boost::math::cyl_bessel_j_prime(v1 * 1, v2 + 0);
+   boost::math::cyl_bessel_j_prime(i, v2 * 1);
+   boost::math::cyl_bessel_i_prime(v1 * 1, v2 + 0);
+   boost::math::cyl_bessel_i_prime(i, v2 * 1);
+   boost::math::cyl_bessel_k_prime(v1 * 1, v2 + 0);
+   boost::math::cyl_bessel_k_prime(i, v2 * 1);
+   boost::math::sph_bessel_prime(i, v2 * 1);
+   boost::math::sph_bessel_prime(i, 1);
+   boost::math::sph_neumann_prime(i, v2 * 1);
+   boost::math::sph_neumann_prime(i, i);
    boost::math::cyl_bessel_j_zero(v1 * 1, i);
    boost::math::cyl_bessel_j_zero(v1 * 1, i, i, oi);
    boost::math::cyl_neumann_zero(v1 * 1, i);
@@ -493,6 +543,8 @@ void instantiate(RealType)
    boost::math::lgamma(v1, pol);
    boost::math::lgamma(v1, &i, pol);
    boost::math::digamma(v1, pol);
+   boost::math::trigamma(v1, pol);
+   boost::math::polygamma(i, v1, pol);
    boost::math::tgamma_ratio(v1, v2, pol);
    boost::math::tgamma_delta_ratio(v1, v2, pol);
    boost::math::factorial<RealType>(i, pol);
@@ -526,6 +578,7 @@ void instantiate(RealType)
    boost::math::ibetac_invb(v1, v2, v3, pol);
    boost::math::gamma_p_derivative(v2, v3, pol);
    boost::math::ibeta_derivative(v1, v2, v3, pol);
+   boost::math::binomial_coefficient<RealType>(i, i, pol);
    boost::math::log1p(v1, pol);
    boost::math::expm1(v1, pol);
    boost::math::cbrt(v1, pol);
@@ -550,9 +603,14 @@ void instantiate(RealType)
    boost::math::ellint_2(v1, v2, pol);
    boost::math::ellint_3(v1, v2, pol);
    boost::math::ellint_3(v1, v2, v3, pol);
+   boost::math::ellint_d(v1, pol);
+   boost::math::ellint_d(v1, v2, pol);
+   boost::math::jacobi_zeta(v1, v2, pol);
+   boost::math::heuman_lambda(v1, v2, pol);
    boost::math::ellint_rc(v1, v2, pol);
    boost::math::ellint_rd(v1, v2, v3, pol);
    boost::math::ellint_rf(v1, v2, v3, pol);
+   boost::math::ellint_rg(v1, v2, v3, pol);
    boost::math::ellint_rj(v1, v2, v3, v1, pol);
    boost::math::jacobi_elliptic(v1, v2, &v1, &v2, pol);
    boost::math::jacobi_cd(v1, v2, pol);
@@ -587,6 +645,18 @@ void instantiate(RealType)
    boost::math::sph_bessel(i, 1, pol);
    boost::math::sph_neumann(i, v2, pol);
    boost::math::sph_neumann(i, i, pol);
+   boost::math::cyl_neumann_prime(v1, v2, pol);
+   boost::math::cyl_neumann_prime(i, v2, pol);
+   boost::math::cyl_bessel_j_prime(v1, v2, pol);
+   boost::math::cyl_bessel_j_prime(i, v2, pol);
+   boost::math::cyl_bessel_i_prime(v1, v2, pol);
+   boost::math::cyl_bessel_i_prime(i, v2, pol);
+   boost::math::cyl_bessel_k_prime(v1, v2, pol);
+   boost::math::cyl_bessel_k_prime(i, v2, pol);
+   boost::math::sph_bessel_prime(i, v2, pol);
+   boost::math::sph_bessel_prime(i, 1, pol);
+   boost::math::sph_neumann_prime(i, v2, pol);
+   boost::math::sph_neumann_prime(i, i, pol);
    boost::math::cyl_bessel_j_zero(v1, i, pol);
    boost::math::cyl_bessel_j_zero(v1, i, i, oi, pol);
    boost::math::cyl_neumann_zero(v1, i, pol);
@@ -643,6 +713,11 @@ void instantiate(RealType)
    boost::math::float_next(v1, pol);
    boost::math::float_prior(v1, pol);
    boost::math::float_distance(v1, v1, pol);
+
+   boost::math::bernoulli_b2n<RealType>(i, pol);
+   boost::math::bernoulli_b2n<RealType>(i, i, &v1, pol);
+   boost::math::tangent_t2n<RealType>(i, pol);
+   boost::math::tangent_t2n<RealType>(i, i, &v1, pol);
 #endif
 #ifdef TEST_GROUP_6
    //
@@ -653,6 +728,8 @@ void instantiate(RealType)
    test::lgamma(v1);
    test::lgamma(v1, &i);
    test::digamma(v1);
+   test::trigamma(v1);
+   test::polygamma(i, v1);
    test::tgamma_ratio(v1, v2);
    test::tgamma_delta_ratio(v1, v2);
    test::factorial<RealType>(i);
@@ -686,6 +763,7 @@ void instantiate(RealType)
    test::ibetac_invb(v1, v2, v3);
    test::gamma_p_derivative(v2, v3);
    test::ibeta_derivative(v1, v2, v3);
+   test::binomial_coefficient<RealType>(i, i);
    (test::fpclassify)(v1);
    (test::isfinite)(v1);
    (test::isnormal)(v1);
@@ -719,9 +797,14 @@ void instantiate(RealType)
    test::ellint_2(v1, v2);
    test::ellint_3(v1, v2);
    test::ellint_3(v1, v2, v3);
+   test::ellint_d(v1);
+   test::ellint_d(v1, v2);
+   test::jacobi_zeta(v1, v2);
+   test::heuman_lambda(v1, v2);
    test::ellint_rc(v1, v2);
    test::ellint_rd(v1, v2, v3);
    test::ellint_rf(v1, v2, v3);
+   test::ellint_rg(v1, v2, v3);
    test::ellint_rj(v1, v2, v3, v1);
    test::jacobi_elliptic(v1, v2, &v1, &v2);
    test::jacobi_cd(v1, v2);
@@ -756,6 +839,18 @@ void instantiate(RealType)
    test::sph_bessel(i, 1);
    test::sph_neumann(i, v2);
    test::sph_neumann(i, i);
+   test::cyl_neumann_prime(v1, v2);
+   test::cyl_neumann_prime(i, v2);
+   test::cyl_bessel_j_prime(v1, v2);
+   test::cyl_bessel_j_prime(i, v2);
+   test::cyl_bessel_i_prime(v1, v2);
+   test::cyl_bessel_i_prime(i, v2);
+   test::cyl_bessel_k_prime(v1, v2);
+   test::cyl_bessel_k_prime(i, v2);
+   test::sph_bessel_prime(i, v2);
+   test::sph_bessel_prime(i, 1);
+   test::sph_neumann_prime(i, v2);
+   test::sph_neumann_prime(i, i);
    test::cyl_bessel_j_zero(v1, i);
    test::cyl_bessel_j_zero(v1, i, i, oi);
    test::cyl_neumann_zero(v1, i);
@@ -832,6 +927,8 @@ void instantiate_mixed(RealType)
    boost::math::lgamma(i);
    boost::math::lgamma(i, &i);
    boost::math::digamma(i);
+   boost::math::trigamma(i);
+   boost::math::polygamma(i, i);
    boost::math::tgamma_ratio(i, l);
    boost::math::tgamma_ratio(fr, lr);
    boost::math::tgamma_delta_ratio(i, s);
@@ -910,12 +1007,21 @@ void instantiate_mixed(RealType)
    boost::math::ellint_3(fr, lr);
    boost::math::ellint_3(s, l, i);
    boost::math::ellint_3(fr, dr, lr);
+   boost::math::ellint_d(i);
+   boost::math::ellint_d(i, l);
+   boost::math::ellint_d(fr, lr);
+   boost::math::jacobi_zeta(i, l);
+   boost::math::jacobi_zeta(fr, lr);
+   boost::math::heuman_lambda(i, l);
+   boost::math::heuman_lambda(fr, lr);
    boost::math::ellint_rc(i, s);
    boost::math::ellint_rc(fr, lr);
    boost::math::ellint_rd(s, i, l);
    boost::math::ellint_rd(fr, lr, dr);
    boost::math::ellint_rf(s, l, i);
    boost::math::ellint_rf(fr, dr, lr);
+   boost::math::ellint_rg(s, l, i);
+   boost::math::ellint_rg(fr, dr, lr);
    boost::math::ellint_rj(i, i, s, l);
    boost::math::ellint_rj(i, fr, dr, lr);
    boost::math::jacobi_cd(i, fr);
@@ -951,6 +1057,18 @@ void instantiate_mixed(RealType)
    boost::math::sph_bessel(i, 1);
    boost::math::sph_neumann(i, lr);
    boost::math::sph_neumann(i, i);
+   boost::math::cyl_neumann_prime(fr, dr);
+   boost::math::cyl_neumann_prime(i, s);
+   boost::math::cyl_bessel_j_prime(fr, lr);
+   boost::math::cyl_bessel_j_prime(i, s);
+   boost::math::cyl_bessel_i_prime(fr, lr);
+   boost::math::cyl_bessel_i_prime(i, s);
+   boost::math::cyl_bessel_k_prime(fr, lr);
+   boost::math::cyl_bessel_k_prime(i, s);
+   boost::math::sph_bessel_prime(i, fr);
+   boost::math::sph_bessel_prime(i, 1);
+   boost::math::sph_neumann_prime(i, lr);
+   boost::math::sph_neumann_prime(i, i);
    boost::math::owens_t(fr, dr);
    boost::math::owens_t(i, s);
 
@@ -962,6 +1080,8 @@ void instantiate_mixed(RealType)
    boost::math::lgamma(i, pol);
    boost::math::lgamma(i, &i, pol);
    boost::math::digamma(i, pol);
+   boost::math::trigamma(i, pol);
+   boost::math::polygamma(i, i, pol);
    boost::math::tgamma_ratio(i, l, pol);
    boost::math::tgamma_ratio(fr, lr, pol);
    boost::math::tgamma_delta_ratio(i, s, pol);
@@ -1035,12 +1155,21 @@ void instantiate_mixed(RealType)
    boost::math::ellint_3(fr, lr, pol);
    boost::math::ellint_3(s, l, i, pol);
    boost::math::ellint_3(fr, dr, lr, pol);
+   boost::math::ellint_d(i, pol);
+   boost::math::ellint_d(i, l, pol);
+   boost::math::ellint_d(fr, lr, pol);
+   boost::math::jacobi_zeta(i, l, pol);
+   boost::math::jacobi_zeta(fr, lr, pol);
+   boost::math::heuman_lambda(i, l, pol);
+   boost::math::heuman_lambda(fr, lr, pol);
    boost::math::ellint_rc(i, s, pol);
    boost::math::ellint_rc(fr, lr, pol);
    boost::math::ellint_rd(s, i, l, pol);
    boost::math::ellint_rd(fr, lr, dr, pol);
    boost::math::ellint_rf(s, l, i, pol);
    boost::math::ellint_rf(fr, dr, lr, pol);
+   boost::math::ellint_rg(s, l, i, pol);
+   boost::math::ellint_rg(fr, dr, lr, pol);
    boost::math::ellint_rj(i, i, s, l, pol);
    boost::math::ellint_rj(i, fr, dr, lr, pol);
    boost::math::jacobi_cd(i, fr, pol);
@@ -1076,6 +1205,18 @@ void instantiate_mixed(RealType)
    boost::math::sph_bessel(i, 1, pol);
    boost::math::sph_neumann(i, lr, pol);
    boost::math::sph_neumann(i, i, pol);
+   boost::math::cyl_neumann_prime(fr, dr, pol);
+   boost::math::cyl_neumann_prime(i, s, pol);
+   boost::math::cyl_bessel_j_prime(fr, lr, pol);
+   boost::math::cyl_bessel_j_prime(i, s, pol);
+   boost::math::cyl_bessel_i_prime(fr, lr, pol);
+   boost::math::cyl_bessel_i_prime(i, s, pol);
+   boost::math::cyl_bessel_k_prime(fr, lr, pol);
+   boost::math::cyl_bessel_k_prime(i, s, pol);
+   boost::math::sph_bessel_prime(i, fr, pol);
+   boost::math::sph_bessel_prime(i, 1, pol);
+   boost::math::sph_neumann_prime(i, lr, pol);
+   boost::math::sph_neumann_prime(i, i, pol);
    boost::math::owens_t(fr, dr, pol);
    boost::math::owens_t(i, s, pol);
 #endif
@@ -1085,6 +1226,8 @@ void instantiate_mixed(RealType)
    test::lgamma(i);
    test::lgamma(i, &i);
    test::digamma(i);
+   test::trigamma(i);
+   test::polygamma(i, i);
    test::tgamma_ratio(i, l);
    test::tgamma_ratio(fr, lr);
    test::tgamma_delta_ratio(i, s);
@@ -1163,12 +1306,21 @@ void instantiate_mixed(RealType)
    test::ellint_3(fr, lr);
    test::ellint_3(s, l, i);
    test::ellint_3(fr, dr, lr);
+   test::ellint_d(i);
+   test::ellint_d(i, l);
+   test::ellint_d(fr, lr);
+   test::jacobi_zeta(i, l);
+   test::jacobi_zeta(fr, lr);
+   test::heuman_lambda(i, l);
+   test::heuman_lambda(fr, lr);
    test::ellint_rc(i, s);
    test::ellint_rc(fr, lr);
    test::ellint_rd(s, i, l);
    test::ellint_rd(fr, lr, dr);
    test::ellint_rf(s, l, i);
    test::ellint_rf(fr, dr, lr);
+   test::ellint_rg(s, l, i);
+   test::ellint_rg(fr, dr, lr);
    test::ellint_rj(i, i, s, l);
    test::ellint_rj(i, fr, dr, lr);
    test::hypot(i, s);
@@ -1192,6 +1344,18 @@ void instantiate_mixed(RealType)
    test::sph_bessel(i, 1);
    test::sph_neumann(i, lr);
    test::sph_neumann(i, i);
+   test::cyl_neumann_prime(fr, dr);
+   test::cyl_neumann_prime(i, s);
+   test::cyl_bessel_j_prime(fr, lr);
+   test::cyl_bessel_j_prime(i, s);
+   test::cyl_bessel_i_prime(fr, lr);
+   test::cyl_bessel_i_prime(i, s);
+   test::cyl_bessel_k_prime(fr, lr);
+   test::cyl_bessel_k_prime(i, s);
+   test::sph_bessel_prime(i, fr);
+   test::sph_bessel_prime(i, 1);
+   test::sph_neumann_prime(i, lr);
+   test::sph_neumann_prime(i, i);
    test::airy_ai(i);
    test::airy_bi(i);
    test::airy_ai_prime(i);
