@@ -1,6 +1,6 @@
 // Copyright 2002 The Trustees of Indiana University.
 
-// Use, modification and distribution is subject to the Boost Software 
+// Use, modification and distribution is subject to the Boost Software
 // License, Version 1.0. (See accompanying file LICENSE_1_0.txt or copy at
 // http://www.boost.org/LICENSE_1_0.txt)
 
@@ -10,10 +10,10 @@
 //           Andrew Lumsdaine
 //  See http://www.boost.org/libs/multi_array for documentation.
 
-// 
+//
 // assign_to_array.cpp - multi_array should be constructible from any other
 // array type in the library.
-// 
+//
 //
 #include "generative_tests.hpp"
 #include "boost/array.hpp"
@@ -33,7 +33,7 @@ bool equal(const ArrayA& A, const ArrayB& B)
   typename ArrayA::const_iterator ia;
   typename ArrayB::const_iterator ib = B.begin();
   for (ia = A.begin(); ia != A.end(); ++ia, ++ib)
-    if (!equal(*ia, *ib))
+    if (!::equal(*ia, *ib))
       return false;
   return true;
 }
@@ -50,7 +50,7 @@ template <typename Array>
 void access(Array& A, const const_array_tag&) {
   typedef boost::multi_array<int,3> array3;
   array3 acopy(A);
-  BOOST_CHECK(equal(acopy,A));
+  BOOST_CHECK(::equal(acopy,A));
   ++tests_run;
 }
 
