@@ -17,8 +17,7 @@
 
 #include "boost/multi_array.hpp"
 
-#define BOOST_INCLUDE_MAIN
-#include "boost/test/test_tools.hpp"
+#include "boost/test/minimal.hpp"
 
 #include "boost/array.hpp"
 #include "boost/type.hpp"
