@@ -1,4 +1,4 @@
-// Copyright (C) 2003, Fernando Luis Cacciola Carballal.
+// Copyright (C) 2003, 2008 Fernando Luis Cacciola Carballal.
 //
 // Use, modification, and distribution is subject to the Boost Software
 // License, Version 1.0. (See accompanying file LICENSE_1_0.txt or copy at
@@ -9,13 +9,20 @@
 // You are welcome to contact the author at:
 //  fernando_cacciola@hotmail.com
 //
+// Revisions:
+// 12 May 2008 (added more swap tests)
+//
 #include<iostream>
 #include<stdexcept>
 #include<string>
 
 #define BOOST_ENABLE_ASSERT_HANDLER
 
-#include "boost/optional.hpp"
+#include "boost/bind/apply.hpp" // Included just to test proper interaction with boost::apply<> as reported by Daniel Wallin
+#include "boost/mpl/bool.hpp"
+#include "boost/mpl/bool_fwd.hpp"  // For mpl::true_ and mpl::false_
+
+#include "boost/optional/optional.hpp"
 
 #ifdef __BORLANDC__
 #pragma hdrstop
@@ -115,12 +122,8 @@ void test_basics( T const* )
   // Assignment
   // T::operator= ( T const& x ) is used to copy new value.
   set_pending_assign( ARG(T) ) ;
-  set_pending_copy  ( ARG(T) ) ;
-  set_pending_dtor  ( ARG(T) ) ;
   oa = ob ;
   check_is_not_pending_assign( ARG(T) ) ;
-  check_is_pending_copy      ( ARG(T) ) ;
-  check_is_pending_dtor      ( ARG(T) ) ;
   check_initialized(oa);
   check_value(oa,b,z);
 
@@ -155,6 +158,95 @@ void test_basics( T const* )
   ob.reset();
   check_is_pending_dtor( ARG(T) ) ;
   check_uninitialized(ob);
+
+}
+
+template<class T>
+void test_conditional_ctor_and_get_valur_or ( T const* )
+{
+  TRACE( std::endl << BOOST_CURRENT_FUNCTION  );
+
+  T a(321);
+
+  T z(123);
+
+  optional<T> const cdef0(false,a);
+
+  optional<T> def0(false,a);
+  optional<T> def1 = boost::make_optional(false,a); //  T is not within boost so ADL won't find make_optional unqualified
+  check_uninitialized(def0);
+  check_uninitialized(def1);
+
+  optional<T> const co0(true,a);
+
+  optional<T> o0(true,a);
+  optional<T> o1 = boost::make_optional(true,a); //  T is not within boost so ADL won't find make_optional unqualified
+
+  check_initialized(o0);
+  check_initialized(o1);
+  check_value(o0,a,z);
+  check_value(o1,a,z);
+
+  T b = def0.get_value_or(z);
+  BOOST_CHECK( b == z ) ;
+
+  b = get_optional_value_or(def0,z);
+  BOOST_CHECK( b == z ) ;
+
+  b = o0.get_value_or(z);
+  BOOST_CHECK( b == a ) ;
+
+  b = get_optional_value_or(o0,z);
+  BOOST_CHECK( b == a ) ;
+
+
+  T const& crz = z ;
+  T&        rz = z ;
+
+  T const& crzz = def0.get_value_or(crz);
+  BOOST_CHECK( crzz == crz ) ;
+
+  T& rzz = def0.get_value_or(rz);
+  BOOST_CHECK( rzz == rz ) ;
+
+  T const& crzzz = get_optional_value_or(cdef0,crz);
+  BOOST_CHECK( crzzz == crz ) ;
+
+  T& rzzz = get_optional_value_or(def0,rz);
+  BOOST_CHECK( rzzz == rz ) ;
+
+  T const& crb = o0.get_value_or(crz);
+  BOOST_CHECK( crb == a ) ;
+
+  T& rb = o0.get_value_or(rz);
+  BOOST_CHECK( rb == b ) ;
+
+  T const& crbb = get_optional_value_or(co0,crz);
+  BOOST_CHECK( crbb == b ) ;
+
+  T const& crbbb = get_optional_value_or(o0,crz);
+  BOOST_CHECK( crbbb == b ) ;
+
+  T& rbb = get_optional_value_or(o0,rz);
+  BOOST_CHECK( rbb == b ) ;
+
+  T& ra = a ;
+
+  optional<T&> defref(false,ra);
+  BOOST_CHECK(!defref);
+
+  optional<T&> ref(true,ra);
+  BOOST_CHECK(!!ref);
+
+  a = T(432);
+
+  BOOST_CHECK( *ref == a ) ;
+
+  T& r1 = defref.get_value_or(z);
+  BOOST_CHECK( r1 == z ) ;
+
+  T& r2 = ref.get_value_or(z);
+  BOOST_CHECK( r2 == a ) ;
 }
 
 //
@@ -199,7 +291,7 @@ void test_uninitialized_access( T const* )
   {
     // This should throw because 'def' is uninitialized
     T const& n = def.get() ;
-    unused_variable(n);
+    boost::ignore_unused(n);
     passed = true ;
   }
   catch (...) {}
@@ -210,7 +302,7 @@ void test_uninitialized_access( T const* )
   {
     // This should throw because 'def' is uninitialized
     T const& n = *def ;
-    unused_variable(n);
+    boost::ignore_unused(n);
     passed = true ;
   }
   catch (...) {}
@@ -220,7 +312,7 @@ void test_uninitialized_access( T const* )
   try
   {
     T v(5) ;
-    unused_variable(v);
+    boost::ignore_unused(v);
     // This should throw because 'def' is uninitialized
     *def = v ;
     passed = true ;
@@ -233,7 +325,7 @@ void test_uninitialized_access( T const* )
   {
     // This should throw because 'def' is uninitialized
     T v = *(def.operator->()) ;
-    unused_variable(v);
+    boost::ignore_unused(v);
     passed = true ;
   }
   catch (...) {}
@@ -617,9 +709,9 @@ void test_relops( T const* )
 {
   TRACE( std::endl << BOOST_CURRENT_FUNCTION   );
 
-  T v0(18);
-  T v1(19);
-  T v2(19);
+  T v0(0);
+  T v1(1);
+  T v2(1);
 
   optional<T> def0 ;
   optional<T> def1 ;
@@ -664,6 +756,33 @@ void test_relops( T const* )
   BOOST_CHECK ( opt1 >  opt0 ) ;
   BOOST_CHECK ( opt1 <= opt2 ) ;
   BOOST_CHECK ( opt1 >= opt0 ) ;
+
+  // Compare against a value directly
+  BOOST_CHECK ( opt0 == v0 ) ;
+  BOOST_CHECK ( opt0 != v1 ) ;
+  BOOST_CHECK ( opt1 == v2 ) ;
+  BOOST_CHECK ( opt0 <  v1 ) ;
+  BOOST_CHECK ( opt1 >  v0 ) ;
+  BOOST_CHECK ( opt1 <= v2 ) ;
+  BOOST_CHECK ( opt1 >= v0 ) ;
+  BOOST_CHECK ( v0 != opt1 ) ;
+  BOOST_CHECK ( v1 == opt2 ) ;
+  BOOST_CHECK ( v0 <  opt1 ) ;
+  BOOST_CHECK ( v1 >  opt0 ) ;
+  BOOST_CHECK ( v1 <= opt2 ) ;
+  BOOST_CHECK ( v1 >= opt0 ) ;
+  BOOST_CHECK (   def0 != v0  ) ;
+  BOOST_CHECK ( !(def0 == v0) ) ;
+  BOOST_CHECK (   def0 <  v0  ) ;
+  BOOST_CHECK ( !(def0 >  v0) ) ;
+  BOOST_CHECK (   def0 <= v0  ) ;
+  BOOST_CHECK ( !(def0 >= v0) ) ;
+  BOOST_CHECK (   v0 != def0  ) ;
+  BOOST_CHECK ( !(v0 == def0) ) ;
+  BOOST_CHECK ( !(v0 <  def0) ) ;
+  BOOST_CHECK (   v0 >  def0  ) ;
+  BOOST_CHECK ( !(v0 <= def0) ) ;
+  BOOST_CHECK (   v0 >= opt0  ) ;
 }
 
 template<class T>
@@ -680,6 +799,10 @@ void test_none( T const* )
   BOOST_CHECK ( def0    == none ) ;
   BOOST_CHECK ( non_def != none ) ;
   BOOST_CHECK ( !def1           ) ;
+  BOOST_CHECK ( !(non_def <  none) ) ;
+  BOOST_CHECK (   non_def >  none  ) ;
+  BOOST_CHECK ( !(non_def <= none) ) ;
+  BOOST_CHECK (   non_def >= none  ) ;
 
   non_def = none ;
   BOOST_CHECK ( !non_def ) ;
@@ -687,22 +810,52 @@ void test_none( T const* )
   test_default_implicit_construction(T(1),none);
 }
 
+template<class T>
+void test_arrow( T const* )
+{
+  TRACE( std::endl << BOOST_CURRENT_FUNCTION   );
+
+  T a(1234);
+
+  optional<T>        oa(a) ;
+  optional<T> const coa(a) ;
+
+  BOOST_CHECK ( coa->V() == 1234 ) ;
+
+  oa->V() = 4321 ;
+
+  BOOST_CHECK (     a.V() = 1234 ) ;
+  BOOST_CHECK ( (*oa).V() = 4321 ) ;
+}
+
 void test_with_builtin_types()
 {
   TRACE( std::endl << BOOST_CURRENT_FUNCTION   );
 
   test_basics( ARG(double) );
+  test_conditional_ctor_and_get_valur_or( ARG(double) );
   test_uninitialized_access( ARG(double) );
   test_no_throwing_swap( ARG(double) );
   test_relops( ARG(double) ) ;
   test_none( ARG(double) ) ;
 }
 
+// MSVC < 11.0 doesn't destroy X when we call ptr->VBase::VBase.
+// Make sure that we work around this bug.
+struct VBase : virtual X
+{
+    VBase(int v) : X(v) {}
+    // MSVC 8.0 doesn't generate this correctly...
+    VBase(const VBase& other) : X(static_cast<const X&>(other)) {}
+};
+
 void test_with_class_type()
 {
   TRACE( std::endl << BOOST_CURRENT_FUNCTION   );
 
   test_basics( ARG(X) );
+  test_basics( ARG(VBase) );
+  test_conditional_ctor_and_get_valur_or( ARG(X) );
   test_direct_value_manip( ARG(X) );
   test_uninitialized_access( ARG(X) );
   test_throwing_direct_init( ARG(X) );
@@ -715,6 +868,7 @@ void test_with_class_type()
   test_throwing_swap( ARG(X) );
   test_relops( ARG(X) ) ;
   test_none( ARG(X) ) ;
+  test_arrow( ARG(X) ) ;
   BOOST_CHECK ( X::count == 0 ) ;
 }
 
@@ -752,40 +906,32 @@ void test_no_implicit_conversions()
   test_no_implicit_conversions_impl(p);
 }
 
-struct A {} ;
-void test_conversions1()
+
+// Test for support for classes with overridden operator&
+class CustomAddressOfClass
 {
-  TRACE( std::endl << BOOST_CURRENT_FUNCTION );
+    int n;
 
-#ifndef BOOST_OPTIONAL_NO_CONVERTING_COPY_CTOR
-  char c = 20 ;
-  optional<char> opt0(c);
-  optional<int> opt1(opt0);
-  BOOST_CHECK(*opt1 == static_cast<int>(c));
-#endif
+public:
+    CustomAddressOfClass() : n(0) {}
+    CustomAddressOfClass(CustomAddressOfClass const& that) : n(that.n) {}
+    explicit CustomAddressOfClass(int m) : n(m) {}
+    int* operator& () { return &n; }
+    bool operator== (CustomAddressOfClass const& that) const { return n == that.n; }
+};
 
-#ifndef BOOST_OPTIONAL_NO_CONVERTING_ASSIGNMENT
-  float f = 21.22f ;
-  double d = f ;
-  optional<float> opt2(f) ;
-  optional<double> opt3 ;
-  opt3 = opt2 ;
-  BOOST_CHECK(*opt3 == d);
-#endif
-}
-
-void test_conversions2()
+void test_custom_addressof_operator()
 {
-  TRACE( std::endl << BOOST_CURRENT_FUNCTION );
+    boost::optional< CustomAddressOfClass > o1(CustomAddressOfClass(10));
+    BOOST_CHECK(!!o1);
+    BOOST_CHECK(o1.get() == CustomAddressOfClass(10));
 
-  char c = 20 ;
-  optional<int> opt(c);
-  BOOST_CHECK( get(opt) == static_cast<int>(c));
+    o1 = CustomAddressOfClass(20);
+    BOOST_CHECK(!!o1);
+    BOOST_CHECK(o1.get() == CustomAddressOfClass(20));
 
-  float f = 21.22f ;
-  optional<double> opt1;
-  opt1 = f ;
-  BOOST_CHECK(*get(&opt1) == static_cast<double>(f));
+    o1 = boost::none;
+    BOOST_CHECK(!o1);
 }
 
 int test_main( int, char* [] )
@@ -795,8 +941,7 @@ int test_main( int, char* [] )
     test_with_class_type();
     test_with_builtin_types();
     test_no_implicit_conversions();
-    test_conversions1();
-    test_conversions2();
+    test_custom_addressof_operator();
   }
   catch ( ... )
   {
