@@ -1,7 +1,7 @@
 /*
  *
  * Copyright (c) 1998-2002
- * Dr John Maddock
+ * John Maddock
  *
  * Use, modification and distribution are subject to the 
  * Boost Software License, Version 1.0. (See accompanying file 
@@ -19,14 +19,34 @@
 
 #define BOOST_REGEX_SOURCE
 
+#include <boost/config.hpp>
 #include <climits>
 #include <stdexcept>
-#ifdef BOOST_REGEX_V3
-#include <boost/regex/v3/fileiter.hpp>
-#else
+#include <string>
+#include <boost/throw_exception.hpp>
 #include <boost/regex/v4/fileiter.hpp>
+#include <boost/regex/v4/regex_workaround.hpp>
+#include <boost/regex/pattern_except.hpp>
+
+#include <cstdio>
+#if defined(BOOST_NO_STDC_NAMESPACE)
+namespace std{
+   using ::sprintf;
+   using ::fseek;
+   using ::fread;
+   using ::ftell;
+   using ::fopen;
+   using ::fclose;
+   using ::FILE;
+   using ::strcpy;
+   using ::strcpy;
+   using ::strcat;
+   using ::strcmp;
+   using ::strlen;
+}
 #endif
 
+
 #ifndef BOOST_REGEX_NO_FILEITER
 
 #if defined(__CYGWIN__) || defined(__CYGWIN32__)
@@ -47,7 +67,7 @@ namespace boost{
 // directories are separated with '\\'
 // and names are insensitive of case
 
-const char* _fi_sep = "\\";
+BOOST_REGEX_DECL const char* _fi_sep = "\\";
 const char* _fi_sep_alt = "/";
 #define BOOST_REGEX_FI_TRANSLATE(c) std::tolower(c)
 
@@ -57,7 +77,7 @@ const char* _fi_sep_alt = "/";
 // directories are separated with '/'
 // and names are sensitive of case
 
-const char* _fi_sep = "/";
+BOOST_REGEX_DECL const char* _fi_sep = "/";
 const char* _fi_sep_alt = _fi_sep;
 #define BOOST_REGEX_FI_TRANSLATE(c) c
 
@@ -67,8 +87,14 @@ const char* _fi_sep_alt = _fi_sep;
 
 void mapfile::open(const char* file)
 {
-   BOOST_RE_GUARD_STACK
-#if defined(__CYGWIN__)||defined(__CYGWIN32__)
+#if defined(BOOST_NO_ANSI_APIS)
+   int filename_size = strlen(file);
+   LPWSTR wide_file = (LPWSTR)_alloca( (filename_size + 1) * sizeof(WCHAR) );
+   if(::MultiByteToWideChar(CP_ACP, 0,  file, filename_size,  wide_file, filename_size + 1) == 0)
+      hfile = INVALID_HANDLE_VALUE;
+   else
+      hfile = CreateFileW(wide_file, GENERIC_READ, FILE_SHARE_READ, 0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
+#elif defined(__CYGWIN__)||defined(__CYGWIN32__)
    char win32file[ MAX_PATH ];
    cygwin_conv_to_win32_path( file, win32file );
    hfile = CreateFileA(win32file, GENERIC_READ, FILE_SHARE_READ, 0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
@@ -84,7 +110,7 @@ void mapfile::open(const char* file)
          hmap = 0;
          hfile = 0;
          std::runtime_error err("Unable to create file mapping.");
-         boost::throw_exception(err);
+         boost::re_detail::raise_runtime_error(err);
       }
       _first = static_cast<const char*>(MapViewOfFile(hmap, FILE_MAP_READ, 0, 0, 0));
       if(_first == 0)
@@ -110,7 +136,6 @@ void mapfile::open(const char* file)
 
 void mapfile::close()
 {
-   BOOST_RE_GUARD_STACK
    if(hfile != INVALID_HANDLE_VALUE)
    {
       UnmapViewOfFile((void*)_first);
@@ -125,7 +150,6 @@ void mapfile::close()
 
 mapfile_iterator& mapfile_iterator::operator = (const mapfile_iterator& i)
 {
-   BOOST_RE_GUARD_STACK
    if(file && node)
       file->unlock(node);
    file = i.file;
@@ -138,7 +162,6 @@ mapfile_iterator& mapfile_iterator::operator = (const mapfile_iterator& i)
 
 mapfile_iterator& mapfile_iterator::operator++ ()
 {
-   BOOST_RE_GUARD_STACK
    if((++offset == mapfile::buf_size) && file)
    {
       ++node;
@@ -151,7 +174,6 @@ mapfile_iterator& mapfile_iterator::operator++ ()
 
 mapfile_iterator mapfile_iterator::operator++ (int)
 {
-   BOOST_RE_GUARD_STACK
    mapfile_iterator temp(*this);
    if((++offset == mapfile::buf_size) && file)
    {
@@ -165,7 +187,6 @@ mapfile_iterator mapfile_iterator::operator++ (int)
 
 mapfile_iterator& mapfile_iterator::operator-- ()
 {
-   BOOST_RE_GUARD_STACK
    if((offset == 0) && file)
    {
       --node;
@@ -180,7 +201,6 @@ mapfile_iterator& mapfile_iterator::operator-- ()
 
 mapfile_iterator mapfile_iterator::operator-- (int)
 {
-   BOOST_RE_GUARD_STACK
    mapfile_iterator temp(*this);
    if((offset == 0) && file)
    {
@@ -196,7 +216,6 @@ mapfile_iterator mapfile_iterator::operator-- (int)
 
 mapfile_iterator operator + (const mapfile_iterator& i, long off)
 {
-   BOOST_RE_GUARD_STACK
    mapfile_iterator temp(i);
    temp += off;
    return temp;
@@ -204,7 +223,6 @@ mapfile_iterator operator + (const mapfile_iterator& i, long off)
 
 mapfile_iterator operator - (const mapfile_iterator& i, long off)
 {
-   BOOST_RE_GUARD_STACK
    mapfile_iterator temp(i);
    temp -= off;
    return temp;
@@ -212,21 +230,18 @@ mapfile_iterator operator - (const mapfile_iterator& i, long off)
 
 mapfile::iterator mapfile::begin()const
 {
-   BOOST_RE_GUARD_STACK
    return mapfile_iterator(this, 0);
 }
 
 mapfile::iterator mapfile::end()const
 {
-   BOOST_RE_GUARD_STACK
    return mapfile_iterator(this, _size);
 }
 
 void mapfile::lock(pointer* node)const
 {
-   BOOST_RE_GUARD_STACK
-   assert(node >= _first);
-   assert(node <= _last);
+   BOOST_ASSERT(node >= _first);
+   BOOST_ASSERT(node <= _last);
    if(node < _last)
    {
       if(*node == 0)
@@ -244,11 +259,23 @@ void mapfile::lock(pointer* node)const
             *p = 0;
             *(reinterpret_cast<int*>(*node)) = 1;
          }
-         std::fseek(hfile, (node - _first) * buf_size, SEEK_SET);
-         if(node == _last - 1)
-            std::fread(*node + sizeof(int), _size % buf_size, 1, hfile);
-         else
-            std::fread(*node + sizeof(int), buf_size, 1, hfile);
+ 
+        std::size_t read_size = 0; 
+        int read_pos = std::fseek(hfile, (node - _first) * buf_size, SEEK_SET); 
+
+        if(0 == read_pos && node == _last - 1) 
+           read_size = std::fread(*node + sizeof(int), _size % buf_size, 1, hfile); 
+        else
+           read_size = std::fread(*node + sizeof(int), buf_size, 1, hfile);
+        if((read_size == 0) || (std::ferror(hfile)))
+        { 
+#ifndef BOOST_NO_EXCEPTIONS 
+           unlock(node);
+           throw std::runtime_error("Unable to read file."); 
+#else 
+           BOOST_REGEX_NOEH_ASSERT((0 == std::ferror(hfile)) && (read_size != 0)); 
+#endif 
+        } 
       }
       else
       {
@@ -265,9 +292,8 @@ void mapfile::lock(pointer* node)const
 
 void mapfile::unlock(pointer* node)const
 {
-   BOOST_RE_GUARD_STACK
-   assert(node >= _first);
-   assert(node <= _last);
+   BOOST_ASSERT(node >= _first);
+   BOOST_ASSERT(node <= _last);
    if(node < _last)
    {
       if(--(*reinterpret_cast<int*>(*node)) == 0)
@@ -279,7 +305,6 @@ void mapfile::unlock(pointer* node)const
 
 long int get_file_length(std::FILE* hfile)
 {
-   BOOST_RE_GUARD_STACK
    long int result;
    std::fseek(hfile, 0, SEEK_END);
    result = std::ftell(hfile);
@@ -290,7 +315,6 @@ long int get_file_length(std::FILE* hfile)
 
 void mapfile::open(const char* file)
 {
-   BOOST_RE_GUARD_STACK
    hfile = std::fopen(file, "rb");
 #ifndef BOOST_NO_EXCEPTIONS
    try{
@@ -325,7 +349,6 @@ void mapfile::open(const char* file)
 
 void mapfile::close()
 {
-   BOOST_RE_GUARD_STACK
    if(hfile != 0)
    {
       pointer* p = _first;
@@ -347,10 +370,51 @@ void mapfile::close()
 
 #endif
 
+inline _fi_find_handle find_first_file(const char* wild,  _fi_find_data& data)
+{
+#ifdef BOOST_NO_ANSI_APIS
+   std::size_t wild_size = std::strlen(wild);
+   LPWSTR wide_wild = (LPWSTR)_alloca( (wild_size + 1) * sizeof(WCHAR) );
+   if (::MultiByteToWideChar(CP_ACP, 0,  wild, wild_size,  wide_wild, wild_size + 1) == 0)
+      return _fi_invalid_handle;
+
+   return FindFirstFileW(wide_wild, &data);
+#else
+   return FindFirstFileA(wild, &data);
+#endif
+}
+
+inline bool find_next_file(_fi_find_handle hf,  _fi_find_data& data)
+{
+#ifdef BOOST_NO_ANSI_APIS
+   return FindNextFileW(hf, &data);
+#else
+   return FindNextFileA(hf, &data);
+#endif
+}
+   
+inline void copy_find_file_result_with_overflow_check(const _fi_find_data& data,  char* path, size_t max_size)
+{
+#ifdef BOOST_NO_ANSI_APIS
+   if (::WideCharToMultiByte(CP_ACP, 0,  data.cFileName, -1,  path, max_size,  NULL, NULL) == 0)
+      re_detail::overflow_error_if_not_zero(1);
+#else
+   re_detail::overflow_error_if_not_zero(re_detail::strcpy_s(path, max_size,  data.cFileName));
+#endif
+}
+
+inline bool is_not_current_or_parent_path_string(const _fi_find_data& data)
+{
+#ifdef BOOST_NO_ANSI_APIS
+   return (std::wcscmp(data.cFileName, L".") && std::wcscmp(data.cFileName, L".."));
+#else
+   return (std::strcmp(data.cFileName, ".") && std::strcmp(data.cFileName, ".."));
+#endif
+}
+
 
 file_iterator::file_iterator()
 {
-   BOOST_RE_GUARD_STACK
    _root = _path = 0;
    ref = 0;
 #ifndef BOOST_NO_EXCEPTIONS
@@ -381,7 +445,6 @@ file_iterator::file_iterator()
 
 file_iterator::file_iterator(const char* wild)
 {
-   BOOST_RE_GUARD_STACK
    _root = _path = 0;
    ref = 0;
 #ifndef BOOST_NO_EXCEPTIONS
@@ -391,38 +454,28 @@ file_iterator::file_iterator(const char* wild)
    BOOST_REGEX_NOEH_ASSERT(_root)
    _path = new char[MAX_PATH];
    BOOST_REGEX_NOEH_ASSERT(_path)
-   std::strcpy(_root, wild);
+   re_detail::overflow_error_if_not_zero(re_detail::strcpy_s(_root, MAX_PATH, wild));
    ptr = _root;
    while(*ptr)++ptr;
    while((ptr > _root) && (*ptr != *_fi_sep) && (*ptr != *_fi_sep_alt))--ptr;
-   #if 0
-   *ptr = 0;
-   std::strcpy(_path, _root);
-   if(*_path == 0)
-      std::strcpy(_path, ".");
-   std::strcat(_path, _fi_sep);
-   ptr = _path + std::strlen(_path);
-   #else
    if((ptr == _root) && ( (*ptr== *_fi_sep) || (*ptr==*_fi_sep_alt) ) )
    {
      _root[1]='\0';
-     std::strcpy(_path, _root);
-     ptr = _path + std::strlen(_path);
+     re_detail::overflow_error_if_not_zero(re_detail::strcpy_s(_path, MAX_PATH, _root));
    }
    else
    {
      *ptr = 0;
-     std::strcpy(_path, _root);
+     re_detail::overflow_error_if_not_zero(re_detail::strcpy_s(_path, MAX_PATH, _root));
      if(*_path == 0)
-       std::strcpy(_path, ".");
-     std::strcat(_path, _fi_sep);
-     ptr = _path + std::strlen(_path);
+       re_detail::overflow_error_if_not_zero(re_detail::strcpy_s(_path, MAX_PATH, "."));
+     re_detail::overflow_error_if_not_zero(re_detail::strcat_s(_path, MAX_PATH, _fi_sep));
    }
-   #endif
+   ptr = _path + std::strlen(_path);
 
    ref = new file_iterator_ref();
    BOOST_REGEX_NOEH_ASSERT(ref)
-   ref->hf = FindFirstFileA(wild, &(ref->_data));
+   ref->hf = find_first_file(wild,  ref->_data);
    ref->count = 1;
 
    if(ref->hf == _fi_invalid_handle)
@@ -432,7 +485,7 @@ file_iterator::file_iterator(const char* wild)
    }
    else
    {
-      std::strcpy(ptr, ref->_data.cFileName);
+      copy_find_file_result_with_overflow_check(ref->_data,  ptr, (MAX_PATH - (ptr - _path)));
       if(ref->_data.dwFileAttributes & _fi_dir)
          next();
    }
@@ -450,7 +503,6 @@ file_iterator::file_iterator(const char* wild)
 
 file_iterator::file_iterator(const file_iterator& other)
 {
-   BOOST_RE_GUARD_STACK
    _root = _path = 0;
    ref = 0;
 #ifndef BOOST_NO_EXCEPTIONS
@@ -460,8 +512,8 @@ file_iterator::file_iterator(const file_iterator& other)
    BOOST_REGEX_NOEH_ASSERT(_root)
    _path = new char[MAX_PATH];
    BOOST_REGEX_NOEH_ASSERT(_path)
-   std::strcpy(_root, other._root);
-   std::strcpy(_path, other._path);
+   re_detail::overflow_error_if_not_zero(re_detail::strcpy_s(_root, MAX_PATH, other._root));
+   re_detail::overflow_error_if_not_zero(re_detail::strcpy_s(_path, MAX_PATH, other._path));
    ptr = _path + (other.ptr - other._path);
    ref = other.ref;
 #ifndef BOOST_NO_EXCEPTIONS
@@ -478,9 +530,8 @@ file_iterator::file_iterator(const file_iterator& other)
 
 file_iterator& file_iterator::operator=(const file_iterator& other)
 {
-   BOOST_RE_GUARD_STACK
-   std::strcpy(_root, other._root);
-   std::strcpy(_path, other._path);
+   re_detail::overflow_error_if_not_zero(re_detail::strcpy_s(_root, MAX_PATH, other._root));
+   re_detail::overflow_error_if_not_zero(re_detail::strcpy_s(_path, MAX_PATH, other._path));
    ptr = _path + (other.ptr - other._path);
    if(--(ref->count) == 0)
    {
@@ -496,7 +547,6 @@ file_iterator& file_iterator::operator=(const file_iterator& other)
 
 file_iterator::~file_iterator()
 {
-   BOOST_RE_GUARD_STACK
    delete[] _root;
    delete[] _path;
    if(--(ref->count) == 0)
@@ -509,7 +559,6 @@ file_iterator::~file_iterator()
 
 file_iterator file_iterator::operator++(int)
 {
-   BOOST_RE_GUARD_STACK
    file_iterator temp(*this);
    next();
    return temp;
@@ -518,13 +567,12 @@ file_iterator file_iterator::operator++(int)
 
 void file_iterator::next()
 {
-   BOOST_RE_GUARD_STACK
    if(ref->hf != _fi_invalid_handle)
    {
       bool cont = true;
       while(cont)
       {
-         cont = FindNextFileA(ref->hf, &(ref->_data));
+         cont = find_next_file(ref->hf, ref->_data);
          if(cont && ((ref->_data.dwFileAttributes & _fi_dir) == 0))
             break;
       }
@@ -537,7 +585,7 @@ void file_iterator::next()
          ptr = _path;
       }
       else
-         std::strcpy(ptr, ref->_data.cFileName);
+         copy_find_file_result_with_overflow_check(ref->_data,  ptr, MAX_PATH - (ptr - _path));
    }
 }
 
@@ -545,7 +593,6 @@ void file_iterator::next()
 
 directory_iterator::directory_iterator()
 {
-   BOOST_RE_GUARD_STACK
    _root = _path = 0;
    ref = 0;
 #ifndef BOOST_NO_EXCEPTIONS
@@ -576,7 +623,6 @@ directory_iterator::directory_iterator()
 
 directory_iterator::directory_iterator(const char* wild)
 {
-   BOOST_RE_GUARD_STACK
    _root = _path = 0;
    ref = 0;
 #ifndef BOOST_NO_EXCEPTIONS
@@ -586,38 +632,30 @@ directory_iterator::directory_iterator(const char* wild)
    BOOST_REGEX_NOEH_ASSERT(_root)
    _path = new char[MAX_PATH];
    BOOST_REGEX_NOEH_ASSERT(_path)
-   std::strcpy(_root, wild);
+   re_detail::overflow_error_if_not_zero(re_detail::strcpy_s(_root, MAX_PATH, wild));
    ptr = _root;
    while(*ptr)++ptr;
    while((ptr > _root) && (*ptr != *_fi_sep) && (*ptr != *_fi_sep_alt))--ptr;
-   #if 0
-   *ptr = 0;
-   std::strcpy(_path, _root);
-   if(*_path == 0)
-      std::strcpy(_path, ".");
-   std::strcat(_path, _fi_sep);
-   ptr = _path + std::strlen(_path);
-   #else
+
    if((ptr == _root) && ( (*ptr== *_fi_sep) || (*ptr==*_fi_sep_alt) ) )
    {
      _root[1]='\0';
-     std::strcpy(_path, _root);
-     ptr = _path + std::strlen(_path);
+     re_detail::overflow_error_if_not_zero(re_detail::strcpy_s(_path, MAX_PATH, _root));
    }
    else
    {
      *ptr = 0;
-     std::strcpy(_path, _root);
+     re_detail::overflow_error_if_not_zero(re_detail::strcpy_s(_path, MAX_PATH, _root));
      if(*_path == 0)
-       std::strcpy(_path, ".");
-     std::strcat(_path, _fi_sep);
-     ptr = _path + std::strlen(_path);
+       re_detail::overflow_error_if_not_zero(re_detail::strcpy_s(_path, MAX_PATH, "."));
+     re_detail::overflow_error_if_not_zero(re_detail::strcat_s(_path, MAX_PATH, _fi_sep));
    }
-   #endif
+   ptr = _path + std::strlen(_path);
+
    ref = new file_iterator_ref();
    BOOST_REGEX_NOEH_ASSERT(ref)
    ref->count = 1;
-   ref->hf = FindFirstFileA(wild, &(ref->_data));
+   ref->hf = find_first_file(wild,  ref->_data);
    if(ref->hf == _fi_invalid_handle)
    {
       *_path = 0;
@@ -625,8 +663,8 @@ directory_iterator::directory_iterator(const char* wild)
    }
    else
    {
-      std::strcpy(ptr, ref->_data.cFileName);
-      if(((ref->_data.dwFileAttributes & _fi_dir) == 0) || (std::strcmp(ref->_data.cFileName, ".") == 0) || (std::strcmp(ref->_data.cFileName, "..") == 0))
+      copy_find_file_result_with_overflow_check(ref->_data,  ptr, MAX_PATH - (ptr - _path));
+      if(((ref->_data.dwFileAttributes & _fi_dir) == 0) || (std::strcmp(ptr, ".") == 0) || (std::strcmp(ptr, "..") == 0))
          next();
    }
 #ifndef BOOST_NO_EXCEPTIONS
@@ -643,7 +681,6 @@ directory_iterator::directory_iterator(const char* wild)
 
 directory_iterator::~directory_iterator()
 {
-   BOOST_RE_GUARD_STACK
    delete[] _root;
    delete[] _path;
    if(--(ref->count) == 0)
@@ -656,7 +693,6 @@ directory_iterator::~directory_iterator()
 
 directory_iterator::directory_iterator(const directory_iterator& other)
 {
-   BOOST_RE_GUARD_STACK
    _root = _path = 0;
    ref = 0;
 #ifndef BOOST_NO_EXCEPTIONS
@@ -666,8 +702,8 @@ directory_iterator::directory_iterator(const directory_iterator& other)
    BOOST_REGEX_NOEH_ASSERT(_root)
    _path = new char[MAX_PATH];
    BOOST_REGEX_NOEH_ASSERT(_path)
-   std::strcpy(_root, other._root);
-   std::strcpy(_path, other._path);
+   re_detail::overflow_error_if_not_zero(re_detail::strcpy_s(_root, MAX_PATH, other._root));
+   re_detail::overflow_error_if_not_zero(re_detail::strcpy_s(_path, MAX_PATH, other._path));
    ptr = _path + (other.ptr - other._path);
    ref = other.ref;
 #ifndef BOOST_NO_EXCEPTIONS
@@ -684,9 +720,8 @@ directory_iterator::directory_iterator(const directory_iterator& other)
 
 directory_iterator& directory_iterator::operator=(const directory_iterator& other)
 {
-   BOOST_RE_GUARD_STACK
-   std::strcpy(_root, other._root);
-   std::strcpy(_path, other._path);
+   re_detail::overflow_error_if_not_zero(re_detail::strcpy_s(_root, MAX_PATH, other._root));
+   re_detail::overflow_error_if_not_zero(re_detail::strcpy_s(_path, MAX_PATH, other._path));
    ptr = _path + (other.ptr - other._path);
    if(--(ref->count) == 0)
    {
@@ -701,7 +736,6 @@ directory_iterator& directory_iterator::operator=(const directory_iterator& othe
 
 directory_iterator directory_iterator::operator++(int)
 {
-   BOOST_RE_GUARD_STACK
    directory_iterator temp(*this);
    next();
    return temp;
@@ -709,16 +743,15 @@ directory_iterator directory_iterator::operator++(int)
 
 void directory_iterator::next()
 {
-   BOOST_RE_GUARD_STACK
    if(ref->hf != _fi_invalid_handle)
    {
       bool cont = true;
       while(cont)
       {
-         cont = FindNextFileA(ref->hf, &(ref->_data));
+         cont = find_next_file(ref->hf, ref->_data);
          if(cont && (ref->_data.dwFileAttributes & _fi_dir))
          {
-            if(std::strcmp(ref->_data.cFileName, ".") && std::strcmp(ref->_data.cFileName, ".."))
+            if(is_not_current_or_parent_path_string(ref->_data))
                break;
          }
       }
@@ -731,7 +764,7 @@ void directory_iterator::next()
          ptr = _path;
       }
       else
-         std::strcpy(ptr, ref->_data.cFileName);
+         copy_find_file_result_with_overflow_check(ref->_data,  ptr, MAX_PATH - (ptr - _path));
    }
 }
 
@@ -748,7 +781,6 @@ struct _fi_priv_data
 
 _fi_priv_data::_fi_priv_data(const char* p)
 {
-   BOOST_RE_GUARD_STACK
    std::strcpy(root, p);
    mask = root;
    while(*mask) ++mask;
@@ -775,7 +807,6 @@ _fi_priv_data::_fi_priv_data(const char* p)
 
 bool iswild(const char* mask, const char* name)
 {
-   BOOST_RE_GUARD_STACK
    while(*mask && *name)
    {
       switch(*mask)
@@ -817,12 +848,17 @@ bool iswild(const char* mask, const char* name)
 
 unsigned _fi_attributes(const char* root, const char* name)
 {
-   BOOST_RE_GUARD_STACK
    char buf[MAX_PATH];
+   // verify that we can not overflow:
+   if(std::strlen(root) + std::strlen(_fi_sep) + std::strlen(name) >= MAX_PATH)
+      return 0;
+   int r;
    if( ( (root[0] == *_fi_sep) || (root[0] == *_fi_sep_alt) ) && (root[1] == '\0') )
-      std::sprintf(buf, "%s%s", root, name);
+      r = (std::sprintf)(buf, "%s%s", root, name);
    else
-      std::sprintf(buf, "%s%s%s", root, _fi_sep, name);
+      r = (std::sprintf)(buf, "%s%s%s", root, _fi_sep, name);
+   if(r < 0)
+      return 0; // sprintf failed
    DIR* d = opendir(buf);
    if(d)
    {
@@ -834,7 +870,6 @@ unsigned _fi_attributes(const char* root, const char* name)
 
 _fi_find_handle _fi_FindFirstFile(const char* lpFileName, _fi_find_data* lpFindFileData)
 {
-   BOOST_RE_GUARD_STACK
    _fi_find_handle dat = new _fi_priv_data(lpFileName);
 
    DIR* h = opendir(dat->root);
@@ -843,6 +878,7 @@ _fi_find_handle _fi_FindFirstFile(const char* lpFileName, _fi_find_data* lpFindF
    {
       if(_fi_FindNextFile(dat, lpFindFileData))
          return dat;
+      closedir(h);
    }
    delete dat;
    return 0;
@@ -850,7 +886,6 @@ _fi_find_handle _fi_FindFirstFile(const char* lpFileName, _fi_find_data* lpFindF
 
 bool _fi_FindNextFile(_fi_find_handle dat, _fi_find_data* lpFindFileData)
 {
-   BOOST_RE_GUARD_STACK
    dirent* d;
    do
    {
@@ -868,7 +903,6 @@ bool _fi_FindNextFile(_fi_find_handle dat, _fi_find_data* lpFindFileData)
 
 bool _fi_FindClose(_fi_find_handle dat)
 {
-   BOOST_RE_GUARD_STACK
    closedir(dat->d);
    delete dat;
    return true;
