@@ -20,6 +20,10 @@
 #include "test_locale.hpp"
 #include <stdarg.h>
 
+#ifdef BOOST_HAS_ICU
+#include <unicode/uloc.h>
+#endif
+
 #ifdef TEST_THREADS
 #include <list>
 #include <boost/thread.hpp>
@@ -33,9 +37,14 @@ int* get_array_data();
 
 int error_count = 0;
 
+#ifndef TEST_THREADS
 #define RUN_TESTS(name) \
    std::cout << "Running test case \"" #name "\".\n";\
    name();
+#else
+#define RUN_TESTS(name) \
+   name();
+#endif
 
 
 void run_tests()
@@ -77,6 +86,19 @@ void run_tests()
 
 int cpp_main(int /*argc*/, char * /*argv*/[])
 {
+#ifdef BOOST_HAS_ICU
+   //
+   // We need to set the default locale used by ICU, 
+   // otherwise some of our tests using equivalence classes fail.
+   //
+   UErrorCode err = U_ZERO_ERROR;
+   uloc_setDefault("en", &err);
+   if(err != U_ZERO_ERROR)
+   {
+      std::cerr << "Unable to set the default ICU locale to \"en\"." << std::endl;
+      return -1;
+   }
+#endif
 #ifdef TEST_THREADS
    try{
       get_array_data();  // initialises data.
@@ -162,19 +184,6 @@ const int* make_array(int first, ...)
    return data;
 }
 
-#ifdef BOOST_NO_EXCEPTIONS
-
-namespace boost{
-
-void throw_exception(std::exception const & e)
-{
-   std::abort();
-}
-
-}
-
-#endif
-
 void test(const char& c, const test_regex_replace_tag& tag)
 {
    do_test(c, tag);
@@ -203,4 +212,24 @@ void test(const wchar_t& c, const test_invalid_regex_tag& tag)
 }
 #endif
 
+#ifdef BOOST_NO_EXCEPTIONS
+namespace boost{
+
+void throw_exception( std::exception const & e )
+{
+   std::cerr << e.what() << std::endl;
+   std::exit(1);
+}
+
+}
+
+int main(int argc, char * argv[])
+{
+   return cpp_main(argc, argv);
+}
+
+#else
+
 #include <boost/test/included/prg_exec_monitor.hpp>
+
+#endif
