@@ -172,27 +172,31 @@ void test_simple_repeats()
    TEST_REGEX_SEARCH("^a{0,1}?$", perl, "aaaaa", match_default, make_array(-2, -2));
    TEST_REGEX_SEARCH("^(?:a){0,1}?$", perl, "aaaaa", match_default, make_array(-2, -2));
    TEST_REGEX_SEARCH("^a(?:bc)?", perl, "abcbc", match_any|match_all, make_array(-2, -2));
+   TEST_REGEX_SEARCH("a}", perl, "a}", match_default, make_array(0, 2, -2, -2));
+   TEST_REGEX_SEARCH("a{12b", perl, "a{12bc", match_default, make_array(0, 5, -2, -2));
+   TEST_INVALID_REGEX("a{b", extended);
+   TEST_INVALID_REGEX("a}b", extended);
+   test_simple_repeats2();
 }
 
 void test_simple_repeats2()
 {
    using namespace boost::regex_constants;
 
-   TEST_INVALID_REGEX("a{}", perl);
-   TEST_INVALID_REGEX("a{", perl);
-   TEST_INVALID_REGEX("a{1", perl);
-   TEST_INVALID_REGEX("a{1,", perl);
-   TEST_INVALID_REGEX("a{1,2", perl);
-   TEST_INVALID_REGEX("a{ 1 , 2 ", perl);
-   TEST_INVALID_REGEX("a{ }", perl);
-   TEST_INVALID_REGEX("a}", perl);
+   TEST_REGEX_SEARCH("a{}", basic, "a{}", match_default, make_array(0, 3, -2, -2));
+   TEST_REGEX_SEARCH("a{", basic, "a{", match_default, make_array(0, 2, -2, -2));
+   TEST_REGEX_SEARCH("a{1", basic, "a{1", match_default, make_array(0, 3, -2, -2));
+   TEST_REGEX_SEARCH("a{1,", basic, "a{1,", match_default, make_array(0, 4, -2, -2));
+   TEST_REGEX_SEARCH("a{1,2", basic, "a{1,2", match_default, make_array(0, 5, -2, -2));
+   TEST_REGEX_SEARCH("a{ 1 , 2", basic, "a{ 1 , 2", match_default, make_array(0, 8, -2, -2));
+   TEST_REGEX_SEARCH("a{ }", basic, "a{ }", match_default, make_array(0, 4, -2, -2));
+   TEST_REGEX_SEARCH("a}", basic, "a}", match_default, make_array(0, 2, -2, -2));
    TEST_INVALID_REGEX("{1}", perl);
-   TEST_INVALID_REGEX("a{b}", perl);
-   TEST_INVALID_REGEX("a{1b}", perl);
-   TEST_INVALID_REGEX("a{1,b}", perl);
-   TEST_INVALID_REGEX("a{1,2v}", perl);
+   TEST_REGEX_SEARCH("a{b}", basic, "a{b}", match_default, make_array(0, 4, -2, -2));
+   TEST_REGEX_SEARCH("a{1b", basic, "a{1b", match_default, make_array(0, 4, -2, -2));
+   TEST_REGEX_SEARCH("a{1,b}", basic, "a{1,b}", match_default, make_array(0, 6, -2, -2));
+   TEST_REGEX_SEARCH("a{1,2v}", basic, "a{1,2v}", match_default, make_array(0, 7, -2, -2));
    TEST_INVALID_REGEX("a{2,1}", perl);
-
    // now try operator \\{\\} for POSIX basic regexes
    TEST_REGEX_SEARCH("a\\{2\\}", basic, "a", match_default, make_array(-2, -2));
    TEST_REGEX_SEARCH("a\\{2\\}", basic|no_intervals, "a{2}", match_default, make_array(0, 4, -2, -2));
@@ -475,5 +479,12 @@ void test_pocessive_repeats()
    TEST_REGEX_SEARCH("x{1,5}+\\w", perl, "xxxxxa", match_default, make_array(0, 6, -2, -2));
    TEST_REGEX_SEARCH("x{1,4}+\\w", perl, "xxxxxa", match_default, make_array(0, 5, -2, -2));
    TEST_REGEX_SEARCH("x{1,3}+\\w", perl, "xxxxxa", match_default, make_array(0, 4, -2, 4, 6, -2, -2));
+   TEST_INVALID_REGEX("\\d+++", perl);
+   TEST_INVALID_REGEX("\\d++*", perl);
+   TEST_INVALID_REGEX("\\d++?", perl);
+   TEST_INVALID_REGEX("\\d++{3}", perl);
+   TEST_INVALID_REGEX("\\d*++", perl);
+   TEST_INVALID_REGEX("\\d?++", perl);
+   TEST_INVALID_REGEX("\\d{1,2}++", perl);
 }
 
