@@ -10,7 +10,7 @@
 
 #include <boost/config.hpp> // msvc 6.0 needs this to suppress warnings
 
-#include <cassert>
+#include <boost/assert.hpp>
 #include <set>
 #include <list>
 #include <vector>
@@ -23,12 +23,14 @@ namespace std{
 } // namespace std
 #endif
 
-#include <boost/limits.hpp>
+#include <boost/integer_traits.hpp>
 #include <boost/serialization/state_saver.hpp>
 #include <boost/serialization/throw_exception.hpp>
 #include <boost/serialization/tracking.hpp>
 
 #define BOOST_ARCHIVE_SOURCE
+// include this to prevent linker errors when the
+// same modules are marked export and import.
 #define BOOST_SERIALIZATION_SOURCE
 
 #include <boost/archive/archive_exception.hpp>
@@ -49,9 +51,8 @@ namespace detail {
 
 class basic_iarchive_impl {
     friend class basic_iarchive;
-    version_type m_archive_library_version;
+    library_version_type m_archive_library_version;
     unsigned int m_flags;
-    const basic_iarchive * m_this;
 
     //////////////////////////////////////////////////////////////////////
     // information about each serialized object loaded
@@ -80,9 +81,18 @@ class basic_iarchive_impl {
 
     //////////////////////////////////////////////////////////////////////
     // used to implement the reset_object_address operation.
-    object_id_type moveable_objects_start;
-    object_id_type moveable_objects_end;
-    object_id_type moveable_objects_recent;
+    struct moveable_objects {
+        object_id_type start;
+        object_id_type end;
+        object_id_type recent;
+        bool is_pointer;
+        moveable_objects() :
+            start(0),
+            end(0),
+            recent(0),
+            is_pointer(false)
+        {}
+    } m_moveable_objects;
 
     void reset_object_address(
         const void * new_address, 
@@ -93,18 +103,18 @@ class basic_iarchive_impl {
     // used by load object to look up class id given basic_serializer
     struct cobject_type
     {
-        const basic_iserializer * bis;
-        const class_id_type class_id;
+        const basic_iserializer * m_bis;
+        const class_id_type m_class_id;
         cobject_type(
-            class_id_type class_id_,
-            const basic_iserializer & bis_
+            std::size_t class_id,
+            const basic_iserializer & bis
         ) : 
-            bis(& bis_),
-            class_id(class_id_)
+            m_bis(& bis),
+            m_class_id(class_id)
         {}
         cobject_type(const cobject_type & rhs) : 
-            bis(rhs.bis),
-            class_id(rhs.class_id)
+            m_bis(rhs.m_bis),
+            m_class_id(rhs.m_class_id)
         {}
         // the following cannot be defined because of the const
         // member.  This will generate a link error if an attempt
@@ -112,7 +122,7 @@ class basic_iarchive_impl {
         cobject_type & operator=(const cobject_type & rhs);
         bool operator<(const cobject_type &rhs) const
         {
-            return *bis < *(rhs.bis);
+            return *m_bis < *(rhs.m_bis);
         }
     };
     typedef std::set<cobject_type> cobject_info_set_type;
@@ -158,23 +168,22 @@ class basic_iarchive_impl {
     //////////////////////////////////////////////////////////////////////
     // address of the most recent object serialized as a poiner
     // whose data itself is now pending serialization
-    void * pending_object;
-    const basic_iserializer * pending_bis;
-    version_type pending_version;
+    struct pending {
+        void * object;
+        const basic_iserializer * bis;
+        version_type version;
+        pending() :
+            object(NULL),
+            bis(NULL),
+            version(0)
+        {}
+    } m_pending;
 
-    basic_iarchive_impl(const basic_iarchive * parent, unsigned int flags) :
+    basic_iarchive_impl(unsigned int flags) :
         m_archive_library_version(BOOST_ARCHIVE_VERSION()),
-        m_flags(flags),
-        m_this(parent),
-        moveable_objects_start(0),
-        moveable_objects_end(0),
-        moveable_objects_recent(0),
-        pending_object(NULL),
-        pending_bis(NULL),
-        pending_version(0)
+        m_flags(flags)
     {}
-    ~basic_iarchive_impl(){}
-    void set_library_version(unsigned int archive_library_version){
+    void set_library_version(library_version_type archive_library_version){
         m_archive_library_version = archive_library_version;
     }
     bool
@@ -200,7 +209,7 @@ class basic_iarchive_impl {
 //public:
     void
     next_object_pointer(void * t){
-        pending_object = t;
+        m_pending.object = t;
     }
     void delete_created_pointers();
     class_id_type register_type(
@@ -214,15 +223,21 @@ class basic_iarchive_impl {
     const basic_pointer_iserializer * load_pointer(
         basic_iarchive & ar,
         void * & t, 
-        const basic_pointer_iserializer * bpis
+        const basic_pointer_iserializer * bpis,
+        const basic_pointer_iserializer * (*finder)(
+            const boost::serialization::extended_type_info & type
+        )
     );
 };
 
 inline void 
 basic_iarchive_impl::reset_object_address(
-    const void * new_address, 
-    const void *old_address
+    void const * const new_address, 
+    void const * const old_address
 ){
+    if(m_moveable_objects.is_pointer)
+        return;
+
     // this code handles a couple of situations.
     // a) where reset_object_address is applied to an untracked object.
     //    In such a case the call is really superfluous and its really an
@@ -235,19 +250,19 @@ basic_iarchive_impl::reset_object_address(
     //    but the code may work anyway.  Naturally, a bad practice on the part
     //    of the programmer but we can't detect it - as above.  So maybe we
     //    can save a few more people from themselves as above.
-    object_id_type i;
-    for(i = moveable_objects_recent; i < moveable_objects_end; ++i){
+    object_id_type i = m_moveable_objects.recent;
+    for(; i < m_moveable_objects.end; ++i){
         if(old_address == object_id_vector[i].address)
             break;
     }
-    for(; i < moveable_objects_end; ++i){
-
+    for(; i < m_moveable_objects.end; ++i){
+        void const * const this_address = object_id_vector[i].address;
         // calculate displacement from this level
         // warning - pointer arithmetic on void * is in herently non-portable
         // but expected to work on all platforms in current usage
-        if(object_id_vector[i].address > old_address){
+        if(this_address > old_address){
             std::size_t member_displacement
-                = reinterpret_cast<std::size_t>(object_id_vector[i].address) 
+                = reinterpret_cast<std::size_t>(this_address) 
                 - reinterpret_cast<std::size_t>(old_address);
             object_id_vector[i].address = reinterpret_cast<void *>(
                 reinterpret_cast<std::size_t>(new_address) + member_displacement
@@ -256,7 +271,7 @@ basic_iarchive_impl::reset_object_address(
         else{
             std::size_t member_displacement
                 = reinterpret_cast<std::size_t>(old_address)
-                - reinterpret_cast<std::size_t>(object_id_vector[i].address); 
+                - reinterpret_cast<std::size_t>(this_address); 
             object_id_vector[i].address = reinterpret_cast<void *>(
                 reinterpret_cast<std::size_t>(new_address) - member_displacement
             );
@@ -289,21 +304,21 @@ inline class_id_type
 basic_iarchive_impl::register_type(
     const basic_iserializer & bis
 ){
-    class_id_type id(static_cast<int>(cobject_info_set.size()));
-    cobject_type co(id, bis);
+    class_id_type cid(cobject_info_set.size());
+    cobject_type co(cid, bis);
     std::pair<cobject_info_set_type::const_iterator, bool>
         result = cobject_info_set.insert(co);
 
     if(result.second){
         cobject_id_vector.push_back(cobject_id(bis));
-        assert(cobject_info_set.size() == cobject_id_vector.size());
+        BOOST_ASSERT(cobject_info_set.size() == cobject_id_vector.size());
     }
-    id = result.first->class_id;
+    cid = result.first->m_class_id;
     // borland complains without this minor hack
-    const int tid = id;
+    const int tid = cid;
     cobject_id & coid = cobject_id_vector[tid];
     coid.bpis_ptr = bis.get_bpis_ptr();
-    return id;
+    return cid;
 }
 
 void
@@ -313,7 +328,7 @@ basic_iarchive_impl::load_preamble(
 ){
     if(! co.initialized){
         if(co.bis_ptr->class_info()){
-            class_id_optional_type cid;
+            class_id_optional_type cid(class_id_type(0));
             load(ar, cid);    // to be thrown away
             load(ar, co.tracking_level);
             load(ar, co.file_version);
@@ -352,10 +367,12 @@ basic_iarchive_impl::load_object(
     void * t,
     const basic_iserializer & bis
 ){
+    m_moveable_objects.is_pointer = false;
+    serialization::state_saver<bool> ss_is_pointer(m_moveable_objects.is_pointer);
     // if its been serialized through a pointer and the preamble's been done
-    if(t == pending_object && & bis == pending_bis){
+    if(t == m_pending.object && & bis == m_pending.bis){
         // read data
-        (bis.load_object_data)(ar, t, pending_version);
+        (bis.load_object_data)(ar, t, m_pending.version);
         return;
     }
 
@@ -366,14 +383,14 @@ basic_iarchive_impl::load_object(
     load_preamble(ar, co);
 
     // save the current move stack position in case we want to truncate it
-    boost::serialization::state_saver<object_id_type> w(moveable_objects_start);
+    boost::serialization::state_saver<object_id_type> ss_start(m_moveable_objects.start);
 
     // note: extra line used to evade borland issue
     const bool tracking = co.tracking_level;
 
     object_id_type this_id;
-    moveable_objects_start =
-    this_id = object_id_vector.size();
+    m_moveable_objects.start =
+    this_id = object_id_type(object_id_vector.size());
 
     // if we tracked this object when the archive was saved
     if(tracking){ 
@@ -384,19 +401,25 @@ basic_iarchive_impl::load_object(
         // add a new enty into the tracking list
         object_id_vector.push_back(aobject(t, cid));
         // and add an entry for this object
-        moveable_objects_end = object_id_vector.size();
+        m_moveable_objects.end = object_id_type(object_id_vector.size());
     }
     // read data
     (bis.load_object_data)(ar, t, co.file_version);
-    moveable_objects_recent = this_id;
+    m_moveable_objects.recent = this_id;
 }
 
 inline const basic_pointer_iserializer *
 basic_iarchive_impl::load_pointer(
     basic_iarchive &ar,
     void * & t,
-    const basic_pointer_iserializer * bpis_ptr
+    const basic_pointer_iserializer * bpis_ptr,
+    const basic_pointer_iserializer * (*finder)(
+        const boost::serialization::extended_type_info & type_
+    )
 ){
+    m_moveable_objects.is_pointer = true;
+    serialization::state_saver<bool> w(m_moveable_objects.is_pointer);
+
     class_id_type cid;
     load(ar, cid);
 
@@ -423,13 +446,13 @@ basic_iarchive_impl::load_pointer(
                 boost::serialization::throw_exception(
                     archive_exception(archive_exception::unregistered_class)
                 );
-            bpis_ptr = m_this->find(*eti);
+            bpis_ptr = (*finder)(*eti);
         }
-        assert(NULL != bpis_ptr);
-        class_id_type new_cid = register_type(bpis_ptr->get_basic_serializer());
+        BOOST_ASSERT(NULL != bpis_ptr);
+        // class_id_type new_cid = register_type(bpis_ptr->get_basic_serializer());
+        BOOST_VERIFY(register_type(bpis_ptr->get_basic_serializer()) == cid);
         int i = cid;
         cobject_id_vector[i].bpis_ptr = bpis_ptr;
-        assert(new_cid == cid);
     }
     int i = cid;
     cobject_id & co = cobject_id_vector[i];
@@ -445,38 +468,41 @@ basic_iarchive_impl::load_pointer(
         return bpis_ptr;
 
     // save state
-    serialization::state_saver<object_id_type> w_start(moveable_objects_start);
+    serialization::state_saver<object_id_type> w_start(m_moveable_objects.start);
+
+    // allocate space on the heap for the object - to be constructed later
+    t = bpis_ptr->heap_allocation();
+    BOOST_ASSERT(NULL != t);
 
     if(! tracking){
         bpis_ptr->load_object_ptr(ar, t, co.file_version);
     }
     else{
-        serialization::state_saver<void *> x(pending_object);
-        serialization::state_saver<const basic_iserializer *> y(pending_bis);
-        serialization::state_saver<version_type> z(pending_version);
+        serialization::state_saver<void *> x(m_pending.object);
+        serialization::state_saver<const basic_iserializer *> y(m_pending.bis);
+        serialization::state_saver<version_type> z(m_pending.version);
 
-        pending_bis = & bpis_ptr->get_basic_serializer();
-        pending_version = co.file_version;
+        m_pending.bis = & bpis_ptr->get_basic_serializer();
+        m_pending.version = co.file_version;
 
         // predict next object id to be created
         const unsigned int ui = object_id_vector.size();
 
-        serialization::state_saver<object_id_type> w_end(moveable_objects_end);
+        serialization::state_saver<object_id_type> w_end(m_moveable_objects.end);
 
-        // because the following operation could move the items
-        // don't use co after this
+        
         // add to list of serialized objects so that we can properly handle
         // cyclic strucures
         object_id_vector.push_back(aobject(t, cid));
 
+        // remember that that the address of these elements could change
+        // when we make another call so don't use the address
         bpis_ptr->load_object_ptr(
-            ar, 
-            object_id_vector[ui].address, 
-            co.file_version
+            ar,
+            t,
+            m_pending.version
         );
-        t = object_id_vector[ui].address;
         object_id_vector[ui].loaded_as_pointer = true;
-        assert(NULL != t);
     }
 
     return bpis_ptr;
@@ -499,17 +525,15 @@ basic_iarchive::next_object_pointer(void *t){
 
 BOOST_ARCHIVE_DECL(BOOST_PP_EMPTY())
 basic_iarchive::basic_iarchive(unsigned int flags) : 
-    pimpl(new basic_iarchive_impl(this, flags))
+    pimpl(new basic_iarchive_impl(flags))
 {}
 
 BOOST_ARCHIVE_DECL(BOOST_PP_EMPTY())
 basic_iarchive::~basic_iarchive()
-{
-    delete pimpl;
-}
+{}
 
 BOOST_ARCHIVE_DECL(void)
-basic_iarchive::set_library_version(unsigned int archive_library_version){
+basic_iarchive::set_library_version(library_version_type archive_library_version){
     pimpl->set_library_version(archive_library_version);
 }
 
@@ -533,9 +557,13 @@ basic_iarchive::load_object(
 BOOST_ARCHIVE_DECL(const basic_pointer_iserializer *)
 basic_iarchive::load_pointer(
     void * &t, 
-    const basic_pointer_iserializer * bpis_ptr
+    const basic_pointer_iserializer * bpis_ptr,
+    const basic_pointer_iserializer * (*finder)(
+        const boost::serialization::extended_type_info & type_
+    )
+
 ){
-    return pimpl->load_pointer(*this, t, bpis_ptr);
+    return pimpl->load_pointer(*this, t, bpis_ptr, finder);
 }
 
 BOOST_ARCHIVE_DECL(void)
@@ -549,7 +577,7 @@ basic_iarchive::delete_created_pointers()
     pimpl->delete_created_pointers();
 }
 
-BOOST_ARCHIVE_DECL(unsigned int) 
+BOOST_ARCHIVE_DECL(boost::archive::library_version_type) 
 basic_iarchive::get_library_version() const{
     return pimpl->m_archive_library_version;
 }
