@@ -15,7 +15,7 @@
 #include <algorithm>
 #include <set>
 #include <utility>
-#include <cassert>
+#include <boost/assert.hpp>
 #include <cstddef> // NULL
 
 #include <boost/config.hpp> // msvc needs this to suppress warning
@@ -25,13 +25,18 @@
 namespace std{ using ::strcmp; }
 #endif
 
-#include <boost/detail/no_exceptions_support.hpp>
+#include <boost/core/no_exceptions_support.hpp>
 #include <boost/serialization/singleton.hpp>
 #include <boost/serialization/force_include.hpp>
 
 #define BOOST_SERIALIZATION_SOURCE
 #include <boost/serialization/extended_type_info.hpp>
 
+#ifdef BOOST_MSVC
+#  pragma warning(push)
+#  pragma warning(disable : 4511 4512)
+#endif
+
 namespace boost { 
 namespace serialization {
 namespace detail {
@@ -47,9 +52,9 @@ struct key_compare
         if(lhs == rhs)
             return false;
         const char * l = lhs->get_key();
-        assert(NULL != l);
+        BOOST_ASSERT(NULL != l);
         const char * r = rhs->get_key();
-        assert(NULL != r);
+        BOOST_ASSERT(NULL != r);
         // performance shortcut
         // shortcut to exploit string pooling
         if(l == r)
@@ -63,52 +68,67 @@ struct key_compare
 
 typedef std::multiset<const extended_type_info *, key_compare> ktmap;
 
+#ifdef BOOST_MSVC
+#  pragma warning(push)
+#  pragma warning(disable : 4511 4512)
+#endif
+
 class extended_type_info_arg : public extended_type_info
 {
     virtual bool
     is_less_than(const extended_type_info & /*rhs*/) const {
-        assert(false);
+        BOOST_ASSERT(false);
         return false;
     };
     virtual bool
     is_equal(const extended_type_info & /*rhs*/) const {
-        assert(false);
+        BOOST_ASSERT(false);
         return false;
     };
+    virtual const char * get_debug_info() const {
+        return get_key();
+    }
+    virtual void * construct(unsigned int /*count*/, ...) const{
+        BOOST_ASSERT(false);
+        return NULL;
+    }
+    virtual void destroy(void const * const /*p*/) const {
+        BOOST_ASSERT(false);
+    }
 public:
     extended_type_info_arg(const char * key) :
-        extended_type_info()
-    {
-        m_key = key;
-    }
+        extended_type_info(0, key)
+    {}
+
     ~extended_type_info_arg(){
-        m_key = NULL;
     }
 };
 
+#ifdef BOOST_MSVC
+#  pragma warning(pop)
+#endif
+
 } // namespace detail
 
 BOOST_SERIALIZATION_DECL(void)  
-extended_type_info::key_register(const char *key) {
-    assert(NULL != key);
-    m_key = key;
+extended_type_info::key_register() const{
+    if(NULL == get_key())
+        return;
     singleton<detail::ktmap>::get_mutable_instance().insert(this);
 }
 
 BOOST_SERIALIZATION_DECL(void)  
-extended_type_info::key_unregister() {
-    assert(NULL != m_key);
+extended_type_info::key_unregister() const{
+    if(NULL == get_key())
+        return;
     if(! singleton<detail::ktmap>::is_destroyed()){
         detail::ktmap & x = singleton<detail::ktmap>::get_mutable_instance();
         detail::ktmap::iterator start = x.lower_bound(this);
         detail::ktmap::iterator end = x.upper_bound(this);
-        assert(start != end);
-
         // remove entry in map which corresponds to this type
         for(;start != end; ++start){
             if(this == *start){
                 x.erase(start);
-                m_key = NULL;
                 break;
             }
         }
@@ -117,7 +137,7 @@ extended_type_info::key_unregister() {
 
 BOOST_SERIALIZATION_DECL(const extended_type_info *) 
 extended_type_info::find(const char *key) {
-    assert(NULL != key);
+    BOOST_ASSERT(NULL != key);
     const detail::ktmap & k = singleton<detail::ktmap>::get_const_instance();
     const detail::extended_type_info_arg eti_key(key);
     const detail::ktmap::const_iterator it = k.find(& eti_key);
@@ -126,28 +146,18 @@ extended_type_info::find(const char *key) {
     return *(it);
 }
 
-struct null_deleter
-{
-    void operator()(void const *) const
-    {
-    }
-};
-
 BOOST_SERIALIZATION_DECL(BOOST_PP_EMPTY())
 extended_type_info::extended_type_info(
-    const unsigned int type_info_key
+    const unsigned int type_info_key,
+    const char * key
 ) :
-    m_this(this, null_deleter()),
     m_type_info_key(type_info_key),
-    m_key(NULL)
+    m_key(key)
 {
 }
 
 BOOST_SERIALIZATION_DECL(BOOST_PP_EMPTY()) 
 extended_type_info::~extended_type_info(){
-    if(NULL == m_key)
-        return;
-    key_unregister();
 }
 
 BOOST_SERIALIZATION_DECL(bool)  
