@@ -1,18 +1,19 @@
 /////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
 // test_diamond.cpp
 
-// (C) Copyright 2002 Vladimir Prus. 
+// (C) Copyright 2002-2009 Vladimir Prus and Robert Ramey. 
 // Use, modification and distribution is subject to the Boost Software
 // License, Version 1.0. (See accompanying file LICENSE_1_0.txt or copy at
 // http://www.boost.org/LICENSE_1_0.txt)
 
-// test of serialization library for diamond intheritence situations
+// test of serialization library for diamond inheritence situations
 
+#include <cstddef> // NULL
 #include <fstream>
 #include <iostream>
 
-#include <cstdio> // remove
 #include <boost/config.hpp>
+#include <cstdio> // remove
 #if defined(BOOST_NO_STDC_NAMESPACE)
 namespace std{ 
     using ::remove;
@@ -20,8 +21,6 @@ namespace std{
 #endif
 
 #include "test_tools.hpp"
-#include <boost/preprocessor/stringize.hpp>
-#include BOOST_PP_STRINGIZE(BOOST_ARCHIVE_TEST)
 
 #include <boost/serialization/map.hpp>
 #include <boost/serialization/utility.hpp>
@@ -31,8 +30,6 @@ namespace std{
 #include <boost/serialization/nvp.hpp>
 #include <boost/serialization/export.hpp>
 
-using namespace boost;
-
 int save_count = 0; // used to detect when base class is saved multiple times
 int load_count = 0; // used to detect when base class is loaded multiple times
 
@@ -48,7 +45,8 @@ public:
     void save(Archive &ar, const unsigned int /* file_version */) const
     {
         std::cout << "Saving base\n";
-        ar << BOOST_SERIALIZATION_NVP(i) << BOOST_SERIALIZATION_NVP(m);
+        ar << BOOST_SERIALIZATION_NVP(i);
+        ar << BOOST_SERIALIZATION_NVP(m);
         ++save_count;
     }
 
@@ -56,7 +54,8 @@ public:
     void load(Archive & ar, const unsigned int /* file_version */)
     {
         std::cout << "Restoring base\n";
-        ar >> BOOST_SERIALIZATION_NVP(i) >> BOOST_SERIALIZATION_NVP(m);
+        ar >> BOOST_SERIALIZATION_NVP(i);
+        ar >> BOOST_SERIALIZATION_NVP(m);
         ++load_count;
     }
 
@@ -66,7 +65,7 @@ public:
     {
         return i == another.i && m == another.m;
     }
-    // make virtual to evade gcc quirk
+    // make polymorphic by marking at least one function virtual
     virtual ~base() {};
 private:
     int i;
@@ -186,19 +185,19 @@ test_main( int /* argc */, char* /* argv */[] )
     {
         test_ostream ofs(testfile);    
         test_oarchive oa(ofs);
-        oa << boost::serialization::make_nvp("bp", bp);
+        oa << BOOST_SERIALIZATION_NVP(bp);
     }
 
     base* bp2;
     {
         test_istream ifs(testfile);
         test_iarchive ia(ifs);
-        ia >> boost::serialization::make_nvp("bp2", bp2);
+        ia >> BOOST_SERIALIZATION_NVP(bp2);
     }
 
-    BOOST_ASSERT(1 == save_count);
-    BOOST_ASSERT(1 == load_count);
-    BOOST_ASSERT(*bp2 == *bp);
+    BOOST_CHECK(1 == save_count);
+    BOOST_CHECK(1 == load_count);
+    BOOST_CHECK(*bp2 == *bp);
     std::remove(testfile);
 
     return EXIT_SUCCESS;
