@@ -12,24 +12,27 @@
 // a) non-intrusive method of implementing serialization
 // b) usage of a non-default constructor
 
+#include <cstddef> // NULL
+#include <cstdio>  // remove()
 #include <fstream>
 #include <cstdlib> // for rand()
-#include <cstdio>  // remove()
-#include <cmath> // for fabs()
-
 #include <boost/config.hpp>
+#include <boost/detail/workaround.hpp>
+#include <boost/limits.hpp>
+#include <boost/math/special_functions/next.hpp>
+
 #if defined(BOOST_NO_STDC_NAMESPACE)
 namespace std{
     using ::rand; 
-    using ::fabs; 
     using ::remove;
+    #if BOOST_WORKAROUND(BOOST_MSVC, >= 1400) && !defined(UNDER_CE)
+        using ::numeric_limits;
+    #endif
 }
 #endif
 
 #include <boost/archive/archive_exception.hpp>
 #include "test_tools.hpp"
-#include <boost/preprocessor/stringize.hpp>
-#include BOOST_PP_STRINGIZE(BOOST_ARCHIVE_TEST)
 
 ///////////////////////////////////////////////////////
 // simple class test - using non-intrusive syntax
@@ -60,6 +63,8 @@ class A
         ar & BOOST_SERIALIZATION_NVP(w);
         ar & BOOST_SERIALIZATION_NVP(x);
     }
+    A(const A & rhs);
+    A & operator=(const A & rhs);
 public:
     static int count;
     const int & get_i() const {
@@ -73,8 +78,8 @@ int A::count = 0;
 
 A::A(int i_) : 
     i(i_),
-    s(std::rand()),
-    t(std::rand()),
+    s(static_cast<signed char>(0xff & std::rand())),
+    t(static_cast<signed char>(0xff & std::rand())),
     u(std::rand()),
     v(std::rand()),
     w((float)std::rand() / std::rand()),
@@ -94,25 +99,25 @@ bool A::operator==(const A &rhs) const
         && t == rhs.t 
         && u == rhs.u 
         && v == rhs.v 
-        && std::fabs(w - rhs.w) <= std::numeric_limits<float>::round_error()
-        && std::fabs(x - rhs.x) <= std::numeric_limits<float>::round_error()
+        && std::abs( boost::math::float_distance(w, rhs.w)) < 2
+        && std::abs( boost::math::float_distance(x, rhs.x)) < 2
     ;
 }
 
 bool A::operator<(const A &rhs) const
 {
-    if(! s == rhs.s )
+    if(! (s == rhs.s) )
         return s < rhs.s;
-    if(! t == rhs.t )
+    if(! (t == rhs.t) )
         return t < rhs.t;
-    if(! u == rhs.u )
+    if(! (u == rhs.u) )
         return t < rhs.u; 
-    if(! v == rhs.v )
+    if(! (v == rhs.v) )
         return t < rhs.v;
-    if(! (std::fabs(w - rhs.w) < std::numeric_limits<float>::round_error() ) )
-        return t < rhs.w; 
-    if(! (std::fabs(x - rhs.x) < std::numeric_limits<float>::round_error() ) )
-        return t < rhs.x;
+    if(std::abs( boost::math::float_distance(w, rhs.w)) > 1)
+        return false;
+    if(std::abs( boost::math::float_distance(x, rhs.x)) > 1)
+        return false;
     return false;
 }
 
@@ -123,7 +128,7 @@ template<class Archive>
 inline void save_construct_data(
     Archive & ar, 
     const A * a, 
-    const unsigned int /* file_version */
+    const BOOST_PFTO unsigned int /* file_version */
 ){
     // variable used for construction
     ar << boost::serialization::make_nvp("i", a->get_i());
@@ -145,7 +150,7 @@ inline void load_construct_data(
 
 void save(const char * testfile){
     test_ostream os(testfile, TEST_STREAM_FLAGS);
-    test_oarchive oa(os);
+    test_oarchive oa(os, TEST_ARCHIVE_FLAGS);
     A a(2);
 
     oa << BOOST_SERIALIZATION_NVP(a);
@@ -162,7 +167,7 @@ void save(const char * testfile){
 }
 void load(const char * testfile){
     test_istream is(testfile, TEST_STREAM_FLAGS);
-    test_iarchive ia(is);
+    test_iarchive ia(is, TEST_ARCHIVE_FLAGS);
 
     A a(4);
     ia >> BOOST_SERIALIZATION_NVP(a);
