@@ -1,21 +1,27 @@
 // Copyright (C) 2001-2003
 // William E. Kempf
+// Copyright (C) 2008 Anthony Williams
 //
-//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
+//  Distributed under the Boost Software License, Version 1.0. (See accompanying
 //  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
+#define BOOST_THREAD_VERSION 2
+#define BOOST_THREAD_PROVIDES_INTERRUPTIONS
+
 #include <boost/thread/detail/config.hpp>
 
-#include <boost/thread/thread.hpp>
+#include <boost/thread/thread_only.hpp>
 #include <boost/thread/xtime.hpp>
 #include <boost/bind.hpp>
 #include <boost/ref.hpp>
 #include <boost/utility.hpp>
 
+#define BOOST_TEST_MODULE Boost.Threads: thread test suite
+
 #include <boost/test/unit_test.hpp>
 
 #define DEFAULT_EXECUTION_MONITOR_TYPE execution_monitor::use_sleep_only
-#include <libs/thread/test/util.inl>
+#include "./util.inl"
 
 int test_value;
 
@@ -27,7 +33,7 @@ void simple_thread()
 void comparison_thread(boost::thread::id parent)
 {
     boost::thread::id const my_id=boost::this_thread::get_id();
-    
+
     BOOST_CHECK(my_id != parent);
     boost::thread::id const my_id2=boost::this_thread::get_id();
     BOOST_CHECK(my_id == my_id2);
@@ -36,14 +42,14 @@ void comparison_thread(boost::thread::id parent)
     BOOST_CHECK(my_id != no_thread_id);
 }
 
-void test_sleep()
+BOOST_AUTO_TEST_CASE(test_sleep)
 {
     boost::xtime xt = delay(3);
     boost::thread::sleep(xt);
 
     // Ensure it's in a range instead of checking actual equality due to time
     // lapse
-    BOOST_CHECK(in_range(xt, 2));
+    BOOST_CHECK(boost::threads::test::in_range(xt, 2));
 }
 
 void do_test_creation()
@@ -54,7 +60,7 @@ void do_test_creation()
     BOOST_CHECK_EQUAL(test_value, 999);
 }
 
-void test_creation()
+BOOST_AUTO_TEST_CASE(test_creation)
 {
     timed_test(&do_test_creation, 1);
 }
@@ -66,14 +72,14 @@ void do_test_id_comparison()
     thrd.join();
 }
 
-void test_id_comparison()
+BOOST_AUTO_TEST_CASE(test_id_comparison)
 {
     timed_test(&do_test_id_comparison, 1);
 }
 
 void interruption_point_thread(boost::mutex* m,bool* failed)
 {
-    boost::mutex::scoped_lock lk(*m);
+    boost::unique_lock<boost::mutex> lk(*m);
     boost::this_thread::interruption_point();
     *failed=true;
 }
@@ -82,7 +88,7 @@ void do_test_thread_interrupts_at_interruption_point()
 {
     boost::mutex m;
     bool failed=false;
-    boost::mutex::scoped_lock lk(m);
+    boost::unique_lock<boost::mutex> lk(m);
     boost::thread thrd(boost::bind(&interruption_point_thread,&m,&failed));
     thrd.interrupt();
     lk.unlock();
@@ -90,14 +96,14 @@ void do_test_thread_interrupts_at_interruption_point()
     BOOST_CHECK(!failed);
 }
 
-void test_thread_interrupts_at_interruption_point()
+BOOST_AUTO_TEST_CASE(test_thread_interrupts_at_interruption_point)
 {
     timed_test(&do_test_thread_interrupts_at_interruption_point, 1);
 }
 
 void disabled_interruption_point_thread(boost::mutex* m,bool* failed)
 {
-    boost::mutex::scoped_lock lk(*m);
+    boost::unique_lock<boost::mutex> lk(*m);
     boost::this_thread::disable_interruption dc;
     boost::this_thread::interruption_point();
     *failed=false;
@@ -107,7 +113,7 @@ void do_test_thread_no_interrupt_if_interrupts_disabled_at_interruption_point()
 {
     boost::mutex m;
     bool failed=true;
-    boost::mutex::scoped_lock lk(m);
+    boost::unique_lock<boost::mutex> lk(m);
     boost::thread thrd(boost::bind(&disabled_interruption_point_thread,&m,&failed));
     thrd.interrupt();
     lk.unlock();
@@ -115,7 +121,7 @@ void do_test_thread_no_interrupt_if_interrupts_disabled_at_interruption_point()
     BOOST_CHECK(!failed);
 }
 
-void test_thread_no_interrupt_if_interrupts_disabled_at_interruption_point()
+BOOST_AUTO_TEST_CASE(test_thread_no_interrupt_if_interrupts_disabled_at_interruption_point)
 {
     timed_test(&do_test_thread_no_interrupt_if_interrupts_disabled_at_interruption_point, 1);
 }
@@ -124,11 +130,11 @@ struct non_copyable_functor:
     boost::noncopyable
 {
     unsigned value;
-    
-    non_copyable_functor():
+
+    non_copyable_functor(): boost::noncopyable(),
         value(0)
     {}
-    
+
     void operator()()
     {
         value=999;
@@ -138,13 +144,13 @@ struct non_copyable_functor:
 void do_test_creation_through_reference_wrapper()
 {
     non_copyable_functor f;
-    
+
     boost::thread thrd(boost::ref(f));
     thrd.join();
     BOOST_CHECK_EQUAL(f.value, 999u);
 }
 
-void test_creation_through_reference_wrapper()
+BOOST_AUTO_TEST_CASE(test_creation_through_reference_wrapper)
 {
     timed_test(&do_test_creation_through_reference_wrapper, 1);
 }
@@ -154,14 +160,14 @@ struct long_running_thread
     boost::condition_variable cond;
     boost::mutex mut;
     bool done;
-    
+
     long_running_thread():
         done(false)
     {}
-    
+
     void operator()()
     {
-        boost::mutex::scoped_lock lk(mut);
+        boost::unique_lock<boost::mutex> lk(mut);
         while(!done)
         {
             cond.wait(lk);
@@ -176,15 +182,15 @@ void do_test_timed_join()
     BOOST_CHECK(thrd.joinable());
     boost::system_time xt=delay(3);
     bool const joined=thrd.timed_join(xt);
-    BOOST_CHECK(in_range(boost::get_xtime(xt), 2));
+    BOOST_CHECK(boost::threads::test::in_range(boost::get_xtime(xt), 2));
     BOOST_CHECK(!joined);
     BOOST_CHECK(thrd.joinable());
     {
-        boost::mutex::scoped_lock lk(f.mut);
+        boost::unique_lock<boost::mutex> lk(f.mut);
         f.done=true;
         f.cond.notify_one();
     }
-    
+
     xt=delay(3);
     bool const joined2=thrd.timed_join(xt);
     boost::system_time const now=boost::get_system_time();
@@ -193,24 +199,24 @@ void do_test_timed_join()
     BOOST_CHECK(!thrd.joinable());
 }
 
-void test_timed_join()
+BOOST_AUTO_TEST_CASE(test_timed_join)
 {
     timed_test(&do_test_timed_join, 10);
 }
 
-
-boost::unit_test_framework::test_suite* init_unit_test_suite(int, char*[])
+BOOST_AUTO_TEST_CASE(test_swap)
 {
-    boost::unit_test_framework::test_suite* test =
-        BOOST_TEST_SUITE("Boost.Threads: thread test suite");
+    boost::thread t(simple_thread);
+    boost::thread t2(simple_thread);
+    boost::thread::id id1=t.get_id();
+    boost::thread::id id2=t2.get_id();
 
-    test->add(BOOST_TEST_CASE(test_sleep));
-    test->add(BOOST_TEST_CASE(test_creation));
-    test->add(BOOST_TEST_CASE(test_id_comparison));
-    test->add(BOOST_TEST_CASE(test_thread_interrupts_at_interruption_point));
-    test->add(BOOST_TEST_CASE(test_thread_no_interrupt_if_interrupts_disabled_at_interruption_point));
-    test->add(BOOST_TEST_CASE(test_creation_through_reference_wrapper));
-    test->add(BOOST_TEST_CASE(test_timed_join));
+    t.swap(t2);
+    BOOST_CHECK(t.get_id()==id2);
+    BOOST_CHECK(t2.get_id()==id1);
 
-    return test;
+    swap(t,t2);
+    BOOST_CHECK(t.get_id()==id1);
+    BOOST_CHECK(t2.get_id()==id2);
 }
+
