@@ -23,6 +23,9 @@ TT_TEST_BEGIN(is_class)
    BOOST_CHECK_INTEGRAL_CONSTANT(::tt::is_class<int[2]>::value, false);
    BOOST_CHECK_INTEGRAL_CONSTANT(::tt::is_class<int const[2]>::value, false);
    BOOST_CHECK_INTEGRAL_CONSTANT(::tt::is_class<int&>::value, false);
+#ifndef BOOST_NO_CXX11_RVALUE_REFERENCES
+   BOOST_CHECK_INTEGRAL_CONSTANT(::tt::is_class<int&&>::value, false);
+#endif
    BOOST_CHECK_INTEGRAL_CONSTANT(::tt::is_class<mf4>::value, false);
    BOOST_CHECK_INTEGRAL_CONSTANT(::tt::is_class<f1>::value, false);
    BOOST_CHECK_INTEGRAL_CONSTANT(::tt::is_class<enum_UDT>::value, false);
